@@ -1,8 +1,11 @@
 #!/usr/bin/python2
 
-import gi
-gi.require_version('NumCosmo', '1.0')
-gi.require_version('NumCosmoMath', '1.0')
+try:
+  import gi
+  gi.require_version('NumCosmo', '1.0')
+  gi.require_version('NumCosmoMath', '1.0')
+except:
+  pass
 
 import ctypes
 from math import *
@@ -45,5 +48,5 @@ for i in range (100):
   x = 1.0 / 99.0 * i
   expx = exp (x)
   odex = os.s.eval (x)
-  print x, expx, odex, fabs ((expx - odex) / expx)
+  print (x, expx, odex, fabs ((expx - odex) / expx))
 
