@@ -1,12 +1,15 @@
 #!/usr/bin/python2
 
-import gi
-gi.require_version('NumCosmo', '1.0')
-gi.require_version('NumCosmoMath', '1.0')
+try:
+  import gi
+  gi.require_version('NumCosmo', '1.0')
+  gi.require_version('NumCosmoMath', '1.0')
+except:
+  pass
 
 from math import *
-from gi.repository import GObject
 import matplotlib.pyplot as plt
+from gi.repository import GObject
 from gi.repository import NumCosmo as Nc
 from gi.repository import NumCosmoMath as Ncm
 
@@ -53,23 +56,27 @@ cosmo.props.Omegax_fit = True
 cosmo.props.w_fit = True
 
 #
-#  A new Distance object optimized to redshift 2.
+#  A new Distance object optimized to redshift 2.5.
 #
-dist = Nc.Distance (zf = 2.0)
+dist = Nc.Distance (zf = 2.5)
 
 #
-#  A new Data object from distance modulus catalogs.
+#  A new Data object from distance modulus catalogs. Type Ia supernovae (snia).
 #  A new Data object from BAO catalogs.
 #
 snia = Nc.DataDistMu.new_from_id (dist, Nc.DataSNIAId.SIMPLE_UNION2_1)
-bao = Nc.data_bao_create (dist, Nc.DataBaoId.A_EISENSTEIN2005)
+bao1 = Nc.data_bao_create (dist, Nc.DataBaoId.RDV_BEUTLER2011)
+bao2 = Nc.data_bao_create (dist, Nc.DataBaoId.EMPIRICAL_FIT_ROSS2015)
+bao3 = Nc.data_bao_create (dist, Nc.DataBaoId.RDV_BOSS_QSO_ATA2017)
 
 #
 #  A new Dataset with snia and bao set.
 #
 dset = Ncm.Dataset ()
 dset.append_data (snia)
-dset.append_data (bao)
+dset.append_data (bao1)
+dset.append_data (bao2)
+dset.append_data (bao3)
 
 #
 #  Creating a Likelihood from the Dataset.
@@ -117,8 +124,8 @@ lhr2d = Ncm.LHRatio2d.new (fit, p1, p2, 1.0e-3)
 #  Calculating the confidence region using the Likelihood ratio test.
 #  Also calculate using the Fisher matrix approach.
 #
-cr_rg = lhr2d.conf_region (0.6826, 300.0, Ncm.FitRunMsgs.SIMPLE)
-fisher_rg = lhr2d.fisher_border (0.6826, 300.0, Ncm.FitRunMsgs.SIMPLE)
+cr_rg = lhr2d.conf_region (0.6827, 300.0, Ncm.FitRunMsgs.SIMPLE)
+fisher_rg = lhr2d.fisher_border (0.6827, 300.0, Ncm.FitRunMsgs.SIMPLE)
 
 cr_p1array = cr_rg.p1.dup_array ()
 cr_p2array = cr_rg.p2.dup_array ()
@@ -130,7 +137,7 @@ fisher_p2array = fisher_rg.p2.dup_array ()
 #  Ploting the confidence regions obtained from both methods.
 #
 
-plt.title ("Confidence regions (%f)" % (cr_rg.clevel * 100.0))
+plt.title ("Confidence regions (%.2f)" % (cr_rg.clevel * 100.0))
 plt.plot (cr_p1array, cr_p2array, 'r', label="Likelihood Ratio")
 plt.plot (fisher_p1array, fisher_p2array, 'b-', label="Fisher Matrix")
 
@@ -139,4 +146,4 @@ plt.ylabel(r'$w$')
 
 plt.legend(loc=4)
 
-plt.savefig ("snia_bao_rg_omegac_w.png")
+plt.savefig ("snia_bao_rg_omegac_w.svg")
