@@ -1,10 +1,13 @@
 #!/usr/bin/python2
 
-import math
-import gi
-gi.require_version('NumCosmo', '1.0')
-gi.require_version('NumCosmoMath', '1.0')
+try:
+  import gi
+  gi.require_version('NumCosmo', '1.0')
+  gi.require_version('NumCosmoMath', '1.0')
+except:
+  pass
 
+import math
 from gi.repository import NumCosmo as Nc
 from gi.repository import NumCosmoMath as Ncm
 
@@ -16,8 +19,9 @@ Ncm.cfg_init ()
 
 #
 #  New homogeneous and isotropic cosmological model NcHICosmoDEXcdm 
+#  with one massive neutrino.
 #
-cosmo = Nc.HICosmo.new_from_name (Nc.HICosmo, "NcHICosmoDEXcdm")
+cosmo = Nc.HICosmo.new_from_name (Nc.HICosmo, "NcHICosmoDEXcdm{'massnu-length':<1>}")
 cosmo.set_reparam (Nc.HICosmoDEReparamCMB.new (cosmo.len ()))
 
 #
@@ -42,6 +46,8 @@ cosmo.orig_param_set (Nc.HICosmoDEParams.T_GAMMA0,  2.72)
 cosmo.orig_param_set (Nc.HICosmoDEParams.OMEGA_B,   0.05)
 cosmo.orig_param_set (Nc.HICosmoDEXCDMParams.W,    -1.10)
 
+cosmo.orig_vparam_set (Nc.HICosmoDEVParams.M, 0, 0.06)
+
 #
 # OO-like
 #
@@ -52,25 +58,26 @@ cosmo.props.Omegax  =  0.70
 cosmo.props.Tgamma0 =  2.72
 cosmo.props.w       = -1.10
 
+massnu_v = Ncm.Vector.new_array ([0.06])
+cosmo.props.massnu  = massnu_v
+
 #
 #  Printing the parameters used.
 #
-print "# Model parameters: ", 
+print ("# Model parameters: ")
 cosmo.params_log_all ()
 
 #
 #  Printing some distances up to redshift 1.0.
 #
-for i in range (0, 10):
-  z  = 1.0 / 9.0 * i
-  cd = cosmo.RH_Mpc () * dist.comoving (cosmo, z)
-  print "% 10.8f % 20.15g" % (z, cd)
+N      = 20
+RH_Mpc = cosmo.RH_Mpc ()
 
-dist = Nc.Distance.new (2000.0)
-
-dist.prepare (cosmo)
-
-print dist.theta100CMB (cosmo)
-print cosmo.zt (5.0)
+for i in range (0, N):
+  z  = 1.0 / (N - 1.0) * i
+  Dc = dist.comoving (cosmo, z)
+  dc = RH_Mpc * Dc
+  
+  print ("% 10.8f % 22.15g [c/H0] % 22.15g [Mpc]" % (z, Dc, dc))
 
 
