@@ -9,15 +9,15 @@
  * Structure containing everything about anisotropy and Fourier power spectra that other modules need to know.
  *
  * Once initialized by spectra_init(), contains a table of all
- * C_l's and P(k) as a function of multipole/wavenumber,
- * mode (scalar/tensor...), type (for C_l's: TT, TE...),
+ * \f$ C_l\f$'s and P(k) as a function of multipole/wavenumber,
+ * mode (scalar/tensor...), type (for \f$ C_l\f$'s: TT, TE...),
  * and pairs of initial conditions (adiabatic, isocurvatures...).
  */
 
 struct spectra {
 
   /** @name - input parameters initialized by user in input module
-      (all other quantitites are computed in this module, given these parameters
+      (all other quantities are computed in this module, given these parameters
       and the content of the 'background', 'perturbs', 'transfers' and
       'primordial' structures) */
 
@@ -51,41 +51,41 @@ struct spectra {
 
   //@{
 
-  int has_tt; /**< do we want C_l^TT ? (T = temperature) */
-  int has_ee; /**< do we want C_l^EE ? (E = E-polarization) */
-  int has_te; /**< do we want C_l^TE ? */
-  int has_bb; /**< do we want C_l^BB ? (B = B-polarization) */
-  int has_pp; /**< do we want C_l^phi-phi ? (phi = CMB lensing potential) */
-  int has_tp; /**< do we want C_l^T-phi ? */
-  int has_ep; /**< do we want C_l^E-phi ? */
-  int has_dd; /**< do we want C_l^dd ? (d = density) */
-  int has_td; /**< do we want C_l^Td ? */
-  int has_pd; /**< do we want C_l^phi-d ? */
-  int has_ll; /**< do we want C_l^l-l ? (l = galaxy lensing potential) */
-  int has_tl; /**< do we want C_l^T-l ? */
-  int has_dl; /**< do we want C_l^d-l ? */
+  int has_tt; /**< do we want \f$ C_l^{TT}\f$? (T = temperature) */
+  int has_ee; /**< do we want \f$ C_l^{EE}\f$? (E = E-polarization) */
+  int has_te; /**< do we want \f$ C_l^{TE}\f$? */
+  int has_bb; /**< do we want \f$ C_l^{BB}\f$? (B = B-polarization) */
+  int has_pp; /**< do we want \f$ C_l^{\phi\phi}\f$? (\f$ \phi \f$ = CMB lensing potential) */
+  int has_tp; /**< do we want \f$ C_l^{T\phi}\f$? */
+  int has_ep; /**< do we want \f$ C_l^{E\phi}\f$? */
+  int has_dd; /**< do we want \f$ C_l^{dd}\f$? (d = density) */
+  int has_td; /**< do we want \f$ C_l^{Td}\f$? */
+  int has_pd; /**< do we want \f$ C_l^{\phi d}\f$? */
+  int has_ll; /**< do we want \f$ C_l^{ll}\f$? (l = galaxy lensing potential) */
+  int has_tl; /**< do we want \f$ C_l^{Tl}\f$? */
+  int has_dl; /**< do we want \f$ C_l^{dl}\f$? */
 
-  int index_ct_tt; /**< index for type C_l^TT */
-  int index_ct_ee; /**< index for type C_l^EE */
-  int index_ct_te; /**< index for type C_l^TE */
-  int index_ct_bb; /**< index for type C_l^BB */
-  int index_ct_pp; /**< index for type C_l^phi-phi */
-  int index_ct_tp; /**< index for type C_l^T-phi */
-  int index_ct_ep; /**< index for type C_l^E-phi */
-  int index_ct_dd; /**< first index for type C_l^dd ((d_size*d_size-(d_size-non_diag)*(d_size-non_diag-1)/2) values) */
-  int index_ct_td; /**< first index for type C_l^Td (d_size values) */
-  int index_ct_pd; /**< first index for type C_l^pd (d_size values) */
-  int index_ct_ll; /**< first index for type C_l^ll ((d_size*d_size-(d_size-non_diag)*(d_size-non_diag-1)/2) values) */
-  int index_ct_tl; /**< first index for type C_l^Tl (d_size values) */
-  int index_ct_dl; /**< first index for type C_l^dl (d_size values) */
+  int index_ct_tt; /**< index for type \f$ C_l^{TT} \f$*/
+  int index_ct_ee; /**< index for type \f$ C_l^{EE} \f$*/
+  int index_ct_te; /**< index for type \f$ C_l^{TE} \f$*/
+  int index_ct_bb; /**< index for type \f$ C_l^{BB} \f$*/
+  int index_ct_pp; /**< index for type \f$ C_l^{\phi\phi} \f$*/
+  int index_ct_tp; /**< index for type \f$ C_l^{T\phi} \f$*/
+  int index_ct_ep; /**< index for type \f$ C_l^{E\phi} \f$*/
+  int index_ct_dd; /**< first index for type \f$ C_l^{dd} \f$((d_size*d_size-(d_size-non_diag)*(d_size-non_diag-1)/2) values) */
+  int index_ct_td; /**< first index for type \f$ C_l^{Td} \f$(d_size values) */
+  int index_ct_pd; /**< first index for type \f$ C_l^{pd} \f$(d_size values) */
+  int index_ct_ll; /**< first index for type \f$ C_l^{ll} \f$((d_size*d_size-(d_size-non_diag)*(d_size-non_diag-1)/2) values) */
+  int index_ct_tl; /**< first index for type \f$ C_l^{Tl} \f$(d_size values) */
+  int index_ct_dl; /**< first index for type \f$ C_l^{dl} \f$(d_size values) */
 
-  int d_size;
+  int d_size;      /**< number of bins for which density Cl's are computed */
 
-  int ct_size; /**< number of C_l types requested */
+  int ct_size; /**< number of \f$ C_l \f$ types requested */
 
   //@}
 
-  /** @name - table of pre-computed C_l values, and related quantitites */
+  /** @name - table of pre-computed C_l values, and related quantities */
 
   //@{
 
@@ -97,19 +97,19 @@ struct spectra {
 
 
   int ** l_max_ct;    /**< last multipole (given as an input) at which
-                         we want to output C_ls for a given mode and type;
+                         we want to output \f$ C_l\f$'s for a given mode and type;
                          l[index_md][l_size[index_md]-1] can be larger
                          than l_max[index_md], in order to ensure a
                          better interpolation with no boundary effects */
 
   int * l_max;    /**< last multipole (given as an input) at which
-                     we want to output C_ls for a given mode (maximized over types);
+                     we want to output \f$ C_l\f$'s for a given mode (maximized over types);
                      l[index_md][l_size[index_md]-1] can be larger
                      than l_max[index_md], in order to ensure a
                      better interpolation with no boundary effects */
 
   int l_max_tot; /**< last multipole (given as an input) at which
-                    we want to output C_ls (maximized over modes and types);
+                    we want to output \f$ C_l\f$'s (maximized over modes and types);
                     l[index_md][l_size[index_md]-1] can be larger
                     than l_max[index_md], in order to ensure a
                     better interpolation with no boundary effects */
@@ -117,29 +117,29 @@ struct spectra {
   double ** cl;   /**< table of anisotropy spectra for each mode, multipole, pair of initial conditions and types, cl[index_md][(index_l * psp->ic_ic_size[index_md] + index_ic1_ic2) * psp->ct_size + index_ct] */
   double ** ddcl; /**< second derivatives of previous table with respect to l, in view of spline interpolation */
 
-  double alpha_II_2_20;
-  double alpha_RI_2_20;
-  double alpha_RR_2_20;
+  double alpha_II_2_20;	/**< parameter describing adiabatic versus isocurvature contribution in mutipole range [2,20] (see Planck parameter papers) */
+  double alpha_RI_2_20;	/**< parameter describing adiabatic versus isocurvature contribution in mutipole range [2,20] (see Planck parameter papers) */
+  double alpha_RR_2_20;	/**< parameter describing adiabatic versus isocurvature contribution in mutipole range [2,20] (see Planck parameter papers) */
 
-  double alpha_II_21_200;
-  double alpha_RI_21_200;
-  double alpha_RR_21_200;
+  double alpha_II_21_200; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [21,200] (see Planck parameter papers) */
+  double alpha_RI_21_200; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [21,200] (see Planck parameter papers) */
+  double alpha_RR_21_200; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [21,200] (see Planck parameter papers) */
 
-  double alpha_II_201_2500;
-  double alpha_RI_201_2500;
-  double alpha_RR_201_2500;
+  double alpha_II_201_2500; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [201,2500] (see Planck parameter papers) */
+  double alpha_RI_201_2500; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [201,2500] (see Planck parameter papers) */
+  double alpha_RR_201_2500; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [201,2500] (see Planck parameter papers) */
 
-  double alpha_II_2_2500;
-  double alpha_RI_2_2500;
-  double alpha_RR_2_2500;
+  double alpha_II_2_2500; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [2,2500] (see Planck parameter papers) */
+  double alpha_RI_2_2500; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [2,2500] (see Planck parameter papers) */
+  double alpha_RR_2_2500; /**< parameter describing adiabatic versus isocurvature contribution in mutipole range [2,2500] (see Planck parameter papers) */
 
-  double alpha_kp;
-  double alpha_k1;
-  double alpha_k2;
+  double alpha_kp; /**< parameter describing adiabatic versus isocurvature contribution at pivot scale (see Planck parameter papers) */
+  double alpha_k1; /**< parameter describing adiabatic versus isocurvature contribution at scale k1 (see Planck parameter papers) */
+  double alpha_k2; /**< parameter describing adiabatic versus isocurvature contribution at scale k2 (see Planck parameter papers) */
 
   //@}
 
-  /** @name - table of pre-computed matter power spectrum P(k) values, and related quantitites */
+  /** @name - table of pre-computed matter power spectrum P(k) values, and related quantities */
 
   //@{
 
@@ -202,6 +202,8 @@ struct spectra {
   int index_tr_theta_dr;       /**< index of decay radiation velocity transfer function */
   int index_tr_theta_ncdm1;    /**< index of first species of non-cold dark matter (massive neutrinos, ...) velocity transfer function */
   int index_tr_theta_tot;      /**< index of total matter velocity transfer function */
+  int index_tr_phi;            /**< index of Bardeen potential phi */
+  int index_tr_psi;            /**< index of Bardeen potential psi */
   int tr_size;                 /**< total number of species in transfer functions */
 
   double * matter_transfer;   /**< Matter transfer functions.
@@ -210,7 +212,7 @@ struct spectra {
                               */
   double * ddmatter_transfer; /**< second derivative of above array with respect to log(tau), for spline interpolation. */
 
-  /* double * LddCl; /\**< density Cl's in the Limber plus thin shell approximation (then, there are no non-diagonal correlations betzeen various shells of different redshifts); depends on index_tau,index_l as: LddCl[index_tau*psp->psp->l_size[psp->index_md_scalars]+index_l] *\/ */
+  /* double * LddCl; /\**< density Cl's in the Limber plus thin shell approximation (then, there are no non-diagonal correlations between various shells of different redshifts); depends on index_tau,index_l as: LddCl[index_tau*psp->psp->l_size[psp->index_md_scalars]+index_l] *\/ */
 
   /* double * LTdCl; /\**< cross (temperature * density) Cl's in the Limber plus thin shell approximation; depends on index_tau,index_l as: LTdCl[index_tau*psp->psp->l_size[psp->index_md_scalars]+index_l] *\/ */
 
@@ -228,7 +230,7 @@ struct spectra {
 };
 
 /*************************************************************************************************************/
-
+/* @cond INCLUDE_WITH_DOXYGEN */
 /*
  * Boilerplate for C++
  */
@@ -406,3 +408,4 @@ extern "C" {
 #endif
 
 #endif
+/* @endcond */
