@@ -60,17 +60,20 @@
 #include "model/nc_hicosmo_qlinear.h"
 #include "model/nc_hicosmo_qspline.h"
 #include "model/nc_hicosmo_lcdm.h"
+#include "model/nc_hicosmo_gcg.h"
+#include "model/nc_hicosmo_idem2.h"
 #include "model/nc_hicosmo_de_xcdm.h"
-#include "model/nc_hicosmo_de_linder.h"
-#include "model/nc_hicosmo_de_pad.h"
-#include "model/nc_hicosmo_de_qe.h"
+#include "model/nc_hicosmo_de_cpl.h"
+#include "model/nc_hicosmo_de_jbp.h"
 #include "model/nc_hicosmo_qgrw.h"
+#include "model/nc_hicosmo_Vexp.h"
 #include "model/nc_hicosmo_de_reparam_ok.h"
 #include "model/nc_hicosmo_de_reparam_cmb.h"
 #include "model/nc_hiprim_power_law.h"
 #include "model/nc_hiprim_atan.h"
 #include "model/nc_hiprim_expc.h"
 #include "model/nc_hiprim_bpl.h"
+#include "model/nc_hiprim_sbpl.h"
 #include "lss/nc_window_tophat.h"
 #include "lss/nc_window_gaussian.h"
 #include "lss/nc_growth_func.h"
@@ -78,7 +81,6 @@
 #include "lss/nc_transfer_func_bbks.h"
 #include "lss/nc_transfer_func_eh.h"
 #include "lss/nc_transfer_func_camb.h"
-#include "lss/nc_transfer_func_pert.h"
 #include "lss/nc_density_profile.h"
 #include "lss/nc_density_profile_nfw.h"
 #include "lss/nc_multiplicity_func.h"
@@ -90,6 +92,7 @@
 #include "lss/nc_multiplicity_func_tinker_mean.h"
 #include "lss/nc_multiplicity_func_tinker_crit.h"
 #include "lss/nc_multiplicity_func_tinker_mean_normalized.h"
+#include "lss/nc_multiplicity_func_crocce.h"
 #include "lss/nc_halo_mass_function.h"
 #include "lss/nc_galaxy_acf.h"
 #include "lss/nc_cluster_mass.h"
@@ -99,6 +102,7 @@
 #include "lss/nc_cluster_mass_benson.h"
 #include "lss/nc_cluster_mass_benson_xray.h"
 #include "lss/nc_cluster_mass_plcl.h"
+#include "lss/nc_cluster_mass_ascaso.h"
 #include "lss/nc_cluster_redshift.h"
 #include "lss/nc_cluster_redshift_nodist.h"
 #include "lss/nc_cluster_photoz_gauss_global.h"
@@ -111,8 +115,11 @@
 #include "lss/nc_cluster_abundance.h"
 #include "lss/nc_cluster_pseudo_counts.h"
 #include "lss/nc_cor_cluster_cmb_lens_limber.h"
+#include "lss/nc_wl_surface_mass_density.h"
+#include "lss/nc_reduced_shear_cluster_mass.h"
 #include "nc_distance.h"
 #include "nc_recomb.h"
+#include "nc_recomb_cbe.h"
 #include "nc_recomb_seager.h"
 #include "nc_hireion.h"
 #include "nc_hireion_camb.h"
@@ -130,19 +137,24 @@
 #include "data/nc_data_bao_dvdv.h"
 #include "data/nc_data_bao_rdv.h"
 #include "data/nc_data_bao_empirical_fit.h"
+#include "data/nc_data_bao_empirical_fit_2d.h"
 #include "data/nc_data_bao_dhr_dar.h"
 #include "data/nc_data_bao_dmr_hr.h"
 #include "data/nc_data_dist_mu.h"
 #include "data/nc_data_cluster_pseudo_counts.h"
+#include "data/nc_data_cluster_counts_box_poisson.h"
 #include "data/nc_data_cmb_shift_param.h"
 #include "data/nc_data_cmb_dist_priors.h"
 #include "data/nc_data_hubble.h"
+#include "data/nc_data_xcor.h"
 #include "xcor/nc_xcor.h"
-#include "xcor/nc_xcor_limber.h"
-#include "xcor/nc_xcor_limber_gal.h"
-#include "xcor/nc_xcor_limber_lensing.h"
-#include "xcor/nc_data_xcor.h"
+#include "xcor/nc_xcor_AB.h"
+#include "xcor/nc_xcor_limber_kernel.h"
+#include "xcor/nc_xcor_limber_kernel_gal.h"
+#include "xcor/nc_xcor_limber_kernel_CMB_lensing.h"
+#include "xcor/nc_xcor_limber_kernel_weak_lensing.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gio/gio.h>
 #ifdef NUMCOSMO_HAVE_FFTW3
 #include <fftw3.h>
@@ -156,6 +168,7 @@
 #ifdef HAVE_EXECINFO_H
 #include <execinfo.h>
 #endif /* HAVE_EXECINFO_H */
+#endif /* NUMCOSMO_GIR_SCAN */
 
 static gchar *numcosmo_path = NULL;
 static gboolean numcosmo_init = FALSE;
@@ -243,8 +256,13 @@ void
 ncm_cfg_init (void)
 {
   const gchar *home;
+
   if (numcosmo_init)
     return;
+
+  if (sizeof (NcmComplex) != sizeof (fftw_complex))
+    g_warning ("NcmComplex is not binary compatible with complex double, expect problems with it!");
+
   home = g_get_home_dir ();
   numcosmo_path = g_build_filename (home, ".numcosmo", NULL);
   if (!g_file_test (numcosmo_path, G_FILE_TEST_EXISTS))
@@ -273,7 +291,14 @@ ncm_cfg_init (void)
 
   gsl_err = gsl_set_error_handler_off ();
 
-#if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 36)
+#ifdef NUMCOSMO_HAVE_FFTW3
+  fftw_set_timelimit (10.0);
+#endif /* NUMCOSMO_HAVE_FFTW3 */
+#ifdef HAVE_FFTW3F
+  fftwf_set_timelimit (10.0);
+#endif /* HAVE_FFTW3F */
+  
+#if !GLIB_CHECK_VERSION(2,36,0)
   g_type_init ();
 #endif
 
@@ -300,7 +325,7 @@ ncm_cfg_init (void)
 
   ncm_cfg_register_obj (NCM_TYPE_POWSPEC);
   ncm_cfg_register_obj (NCM_TYPE_POWSPEC_FILTER);
-  
+
   ncm_cfg_register_obj (NCM_TYPE_MODEL);
   ncm_cfg_register_obj (NCM_TYPE_MODEL_CTRL);
   ncm_cfg_register_obj (NCM_TYPE_MODEL_BUILDER);
@@ -314,25 +339,34 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NCM_TYPE_FIT_ESMCMC_WALKER_STRETCH);
 
   ncm_cfg_register_obj (NCM_TYPE_DATA);
-  
+
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QCONST);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QLINEAR);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QSPLINE);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QSPLINE_CONT_PRIOR);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_LCDM);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_GCG);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_IDEM2);  
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_XCDM);
-  ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_LINDER);
-  ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_PAD);
-  ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_QE);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_CPL);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_JBP);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QGRW);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_VEXP);
 
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_REPARAM_OK);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_REPARAM_CMB);
 
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_GCG_REPARAM_OK);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_GCG_REPARAM_CMB);
+  
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_IDEM2_REPARAM_OK);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_IDEM2_REPARAM_CMB);
+    
   ncm_cfg_register_obj (NC_TYPE_HIPRIM_POWER_LAW);
   ncm_cfg_register_obj (NC_TYPE_HIPRIM_ATAN);
   ncm_cfg_register_obj (NC_TYPE_HIPRIM_EXPC);
   ncm_cfg_register_obj (NC_TYPE_HIPRIM_BPL);
+  ncm_cfg_register_obj (NC_TYPE_HIPRIM_SBPL);
 
   ncm_cfg_register_obj (NC_TYPE_CBE_PRECISION);
 
@@ -346,7 +380,6 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_TRANSFER_FUNC_BBKS);
   ncm_cfg_register_obj (NC_TYPE_TRANSFER_FUNC_EH);
   ncm_cfg_register_obj (NC_TYPE_TRANSFER_FUNC_CAMB);
-  ncm_cfg_register_obj (NC_TYPE_TRANSFER_FUNC_PERT);
 
   ncm_cfg_register_obj (NC_TYPE_DENSITY_PROFILE);
   ncm_cfg_register_obj (NC_TYPE_DENSITY_PROFILE_NFW);
@@ -360,6 +393,7 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_MULTIPLICITY_FUNC_TINKER_MEAN);
   ncm_cfg_register_obj (NC_TYPE_MULTIPLICITY_FUNC_TINKER_CRIT);
   ncm_cfg_register_obj (NC_TYPE_MULTIPLICITY_FUNC_TINKER_MEAN_NORMALIZED);
+  ncm_cfg_register_obj (NC_TYPE_MULTIPLICITY_FUNC_CROCCE);	
 
   ncm_cfg_register_obj (NC_TYPE_HALO_MASS_FUNCTION);
 
@@ -392,9 +426,12 @@ ncm_cfg_init (void)
 
   ncm_cfg_register_obj (NC_TYPE_COR_CLUSTER_CMB_LENS_LIMBER);
 
+  ncm_cfg_register_obj (NC_TYPE_WL_SURFACE_MASS_DENSITY);
+
   ncm_cfg_register_obj (NC_TYPE_DISTANCE);
 
   ncm_cfg_register_obj (NC_TYPE_RECOMB);
+  ncm_cfg_register_obj (NC_TYPE_RECOMB_CBE);
   ncm_cfg_register_obj (NC_TYPE_RECOMB_SEAGER);
 
   ncm_cfg_register_obj (NC_TYPE_HIREION);
@@ -418,6 +455,7 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_DVDV);
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_RDV);
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_EMPIRICAL_FIT);
+  ncm_cfg_register_obj (NC_TYPE_DATA_BAO_EMPIRICAL_FIT_2D);
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_DHR_DAR);
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_DMR_HR);
 
@@ -425,21 +463,25 @@ ncm_cfg_init (void)
 
   ncm_cfg_register_obj (NC_TYPE_DATA_HUBBLE);
 
+  ncm_cfg_register_obj (NC_TYPE_DATA_CLUSTER_COUNTS_BOX_POISSON);
   ncm_cfg_register_obj (NC_TYPE_DATA_CLUSTER_PSEUDO_COUNTS);
 
   ncm_cfg_register_obj (NC_TYPE_DATA_CMB_SHIFT_PARAM);
   ncm_cfg_register_obj (NC_TYPE_DATA_CMB_DIST_PRIORS);
 
   ncm_cfg_register_obj (NC_TYPE_XCOR);
-  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER);
-  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_GAL);
-  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_LENSING);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_KERNEL);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_KERNEL_GAL);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_KERNEL_CMB_LENSING);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_KERNEL_WEAK_LENSING);
   ncm_cfg_register_obj (NC_TYPE_DATA_XCOR);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_AB);
+
 
   _nc_hicosmo_register_functions ();
   _nc_hicosmo_de_register_functions ();
   _nc_hireion_register_functions ();
-    _nc_distance_register_functions ();
+  _nc_distance_register_functions ();
   _nc_planck_fi_cor_tt_register_functions ();
 
   numcosmo_init = TRUE;
@@ -469,7 +511,7 @@ static uint nreg_model = 0;
 void
 ncm_cfg_register_obj (GType obj)
 {
-#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 34))
+#if GLIB_CHECK_VERSION(2,34,0)
   g_type_ensure (obj);
 #endif /* GLIB >= 2.34*/
   gpointer obj_class = g_type_class_ref (obj);
@@ -927,10 +969,10 @@ G_LOCK_DEFINE_STATIC (fftw_plan_lock);
  * FIXME
  *
  */
-void 
+void
 ncm_cfg_lock_plan_fftw (void)
 {
-  G_LOCK (fftw_plan_lock);  
+  G_LOCK (fftw_plan_lock);
 }
 
 /**
@@ -939,10 +981,10 @@ ncm_cfg_lock_plan_fftw (void)
  * FIXME
  *
  */
-void 
+void
 ncm_cfg_unlock_plan_fftw (void)
 {
-  G_UNLOCK (fftw_plan_lock);  
+  G_UNLOCK (fftw_plan_lock);
 }
 
 /**
@@ -964,14 +1006,14 @@ ncm_cfg_load_fftw_wisdom (const gchar *filename, ...)
   g_assert (numcosmo_init);
 
   G_LOCK (fftw_saveload_lock);
-  
+
   va_start (ap, filename);
   file = g_strdup_vprintf (filename, ap);
   va_end (ap);
 
   g_free (file);
   file = g_strdup ("ncm_cfg_wisdom"); /* overwrite, unifying wisdom */
-  
+
   file_ext      = g_strdup_printf ("%s.fftw3", file);
   full_filename = g_build_filename (numcosmo_path, file_ext, NULL);
 
@@ -984,7 +1026,7 @@ ncm_cfg_load_fftw_wisdom (const gchar *filename, ...)
 #ifdef HAVE_FFTW3F
   g_free (file_ext);
   g_free (full_filename);
-  
+
   file_ext      = g_strdup_printf ("%s.fftw3f", file);
   full_filename = g_build_filename (numcosmo_path, file_ext, NULL);
 
@@ -1000,7 +1042,7 @@ ncm_cfg_load_fftw_wisdom (const gchar *filename, ...)
   g_free (full_filename);
 
   G_UNLOCK (fftw_saveload_lock);
-  
+
   return ret;
 }
 
@@ -1022,7 +1064,7 @@ ncm_cfg_save_fftw_wisdom (const gchar *filename, ...)
 
   g_assert (numcosmo_init);
 
-G_LOCK (fftw_saveload_lock);
+  G_LOCK (fftw_saveload_lock);
 
   va_start (ap, filename);
   file = g_strdup_vprintf (filename, ap);
@@ -1039,19 +1081,19 @@ G_LOCK (fftw_saveload_lock);
 #ifdef HAVE_FFTW3F
   g_free (file_ext);
   g_free (full_filename);
-  
+
   file_ext      = g_strdup_printf ("%s.fftw3f", file);
   full_filename = g_build_filename (numcosmo_path, file_ext, NULL);
 
   fftwf_export_wisdom_to_filename (full_filename);
 #endif
-  
+
   g_free (file);
   g_free (file_ext);
   g_free (full_filename);
 
   G_UNLOCK (fftw_saveload_lock);
-  
+
   return TRUE;
 }
 #endif /* NUMCOSMO_HAVE_FFTW3 */
@@ -1250,290 +1292,6 @@ ncm_cfg_save_spline (const gchar *filename, NcmSpline *s, ...)
   return TRUE;
 }
 
-/**
- * ncm_cfg_load_vector: (skip)
- * @filename: FIXME
- * @v: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-gboolean
-ncm_cfg_load_vector (const gchar *filename, gsl_vector *v, ...)
-{
-  va_list ap;
-
-  va_start (ap, v);
-  FILE *F = ncm_cfg_vfopen (filename, "r", ap);
-  va_end (ap);
-
-  if (F == NULL)
-    return FALSE;
-  gsl_vector_fread (F, v);
-  fclose (F);
-  return TRUE;
-}
-
-/**
- * ncm_cfg_save_vector: (skip)
- * @filename: FIXME
- * @v: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-gboolean
-ncm_cfg_save_vector (const gchar *filename, gsl_vector *v, ...)
-{
-  va_list ap;
-
-  va_start(ap, v);
-  FILE *F = ncm_cfg_vfopen (filename, "w", ap);
-  va_end (ap);
-
-  if (F == NULL)
-    return FALSE;
-  gsl_vector_fwrite (F, v);
-  fclose (F);
-  return TRUE;
-}
-
-/**
- * ncm_cfg_load_matrix: (skip)
- * @filename: FIXME
- * @M: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-gboolean
-ncm_cfg_load_matrix (const gchar *filename, gsl_matrix *M, ...)
-{
-  va_list ap;
-
-  va_start (ap, M);
-  FILE *F = ncm_cfg_vfopen (filename, "r", ap);
-  va_end (ap);
-
-  if (F == NULL)
-    return FALSE;
-  gsl_matrix_fread (F, M);
-  fclose (F);
-  return TRUE;
-}
-
-/**
- * ncm_cfg_save_matrix: (skip)
- * @filename: FIXME
- * @M: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-gboolean
-ncm_cfg_save_matrix (const gchar *filename, gsl_matrix *M, ...)
-{
-  va_list ap;
-
-  va_start (ap, M);
-  FILE *F = ncm_cfg_vfopen (filename, "w", ap);
-  va_end (ap);
-
-  if (F == NULL)
-    return FALSE;
-  gsl_matrix_fwrite (F, M);
-  fclose (F);
-  return TRUE;
-}
-
-#define load_save_vector_matrix(typen) \
-gboolean \
-ncm_cfg_load_vector_##typen (gchar *filename, gsl_vector_##typen *v, ...) \
-{ \
-va_list ap; \
-va_start (ap, v); \
-FILE *F = ncm_cfg_vfopen (filename, "r", ap); \
-va_end (ap); \
-if (F == NULL) \
-return FALSE; \
-gsl_vector_##typen##_fread (F, v); \
-fclose (F); \
-return TRUE; \
-} \
-gboolean \
-ncm_cfg_save_vector_##typen (gchar *filename, gsl_vector_##typen *v, ...) \
-{ \
-va_list ap; \
-va_start(ap, v); \
-FILE *F = ncm_cfg_vfopen (filename, "w", ap); \
-va_end (ap); \
-if (F == NULL) \
-return FALSE; \
-gsl_vector_##typen##_fwrite (F, v); \
-fclose (F); \
-return TRUE; \
-} \
-gboolean \
-ncm_cfg_load_matrix_##typen (gchar *filename, gsl_matrix_##typen *M, ...) \
-{ \
-va_list ap; \
-va_start(ap, M); \
-FILE *F = ncm_cfg_vfopen (filename, "r", ap); \
-va_end (ap); \
-if (F == NULL) \
-return FALSE; \
-gsl_matrix_##typen##_fread (F, M); \
-fclose (F); \
-return TRUE; \
-} \
-gboolean \
-ncm_cfg_save_matrix_##typen (gchar *filename, gsl_matrix_##typen *M, ...) \
-{ \
-va_list ap; \
-va_start(ap, M); \
-FILE *F = ncm_cfg_vfopen (filename, "w", ap); \
-va_end (ap); \
-if (F == NULL) \
-return FALSE; \
-gsl_matrix_##typen##_fwrite (F, M); \
-fclose (F); \
-return TRUE; \
-} \
-
-/**
- * ncm_cfg_load_vector_int: (skip)
- * @filename: FIXME
- * @v: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-/**
- * ncm_cfg_save_vector_int: (skip)
- * @filename: FIXME
- * @v: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-/**
- * ncm_cfg_load_matrix_int: (skip)
- * @filename: FIXME
- * @M: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-/**
- * ncm_cfg_save_matrix_int: (skip)
- * @filename: FIXME
- * @M: FIXME
- * @...: FIXME
- *
- * FIXME
- *
- * Returns: FIXME
- */
-load_save_vector_matrix(int)
-
-/**
-   * ncm_cfg_load_vector_float: (skip)
-   * @filename: FIXME
-   * @v: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-/**
-   * ncm_cfg_save_vector_float: (skip)
-   * @filename: FIXME
-   * @v: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-/**
-   * ncm_cfg_load_matrix_float: (skip)
-   * @filename: FIXME
-   * @M: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-/**
-   * ncm_cfg_save_matrix_float: (skip)
-   * @filename: FIXME
-   * @M: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-load_save_vector_matrix(float)
-
-/**
-   * ncm_cfg_load_vector_complex: (skip)
-   * @filename: FIXME
-   * @v: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-/**
-   * ncm_cfg_save_vector_complex: (skip)
-   * @filename: FIXME
-   * @v: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-/**
-   * ncm_cfg_load_matrix_complex: (skip)
-   * @filename: FIXME
-   * @M: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-/**
-   * ncm_cfg_save_matrix_complex: (skip)
-   * @filename: FIXME
-   * @M: FIXME
-   * @...: FIXME
-   *
-   * FIXME
-   *
-   * Returns: FIXME
-   */
-load_save_vector_matrix(complex)
-
 /**
  * ncm_cfg_get_data_filename:
  * @filename: filename to search in the data path.
@@ -1698,18 +1456,30 @@ ncm_cfg_array_to_variant (GArray *a, const GVariantType *etype)
   return g_variant_ref_sink (vvar);
 }
 
-guint fftw_default_flags = FFTW_PATIENT; /* FFTW_ESTIMATE, FFTW_MEASURE, FFTW_PATIENT, FFTW_EXHAUSTIVE */
+gdouble fftw_default_timeout = 60.0;
+
+#ifdef NUMCOSMO_HAVE_FFTW3
+guint fftw_default_flags = FFTW_MEASURE; /* FFTW_ESTIMATE, FFTW_MEASURE, FFTW_PATIENT, FFTW_EXHAUSTIVE */
 
 /**
  * ncm_cfg_set_fftw_default_flag:
- * @flag: a FFTW library flag.
+ * @flag: a FFTW library flag
+ * @timeout: planner time out in seconds
  *
  * Sets the default FFTW flag (FFTW_ESTIMATE, FFTW_MEASURE, FFTW_PATIENT, FFTW_EXHAUSTIVE)
- * to be used when building plans.
+ * to be used when building plans. The variable @timeout sets the maximum time spended on
+ * planners. 
  *
  */
 void
-ncm_cfg_set_fftw_default_flag (guint flag)
+ncm_cfg_set_fftw_default_flag (guint flag, const gdouble timeout)
 {
   fftw_default_flags = flag;
+  fftw_set_timelimit (10.0);
+#ifdef HAVE_FFTW3F
+  fftwf_set_timelimit (10.0);
+#endif /* HAVE_FFTW3F */
 }
+#else
+guint fftw_default_flags = 0;
+#endif /* NUMCOSMO_HAVE_FFTW3 */
