@@ -843,15 +843,52 @@ ncm_mset_is_subset (NcmMSet *mset, NcmMSet *sub_mset)
 
   for (i = 0; i < sub_nmodels; i++)
   {
-    NcmModelID mid = ncm_mset_get_mid_array_pos (sub_mset, i);
+    NcmModelID mid      = ncm_mset_get_mid_array_pos (sub_mset, i);
     NcmModel *sub_model = ncm_mset_peek_array_pos (sub_mset, i);
-    NcmModel *model = ncm_mset_peek (mset, mid);
+    NcmModel *model     = ncm_mset_peek (mset, mid);
     if (sub_model != model)
       return FALSE;
   }
   return TRUE;
 }
 
+/**
+ * ncm_mset_cmp_all:
+ * @mset0: a #NcmMSet
+ * @mset1: a #NcmMSet
+ *
+ * FIXME
+ *
+ * Returns: FIXME
+ */
+gint
+ncm_mset_cmp_all (NcmMSet *mset0, NcmMSet *mset1)
+{
+  guint sub_nmodels0 = ncm_mset_nmodels (mset0);
+  guint sub_nmodels1 = ncm_mset_nmodels (mset1);
+  guint i;
+
+  if (sub_nmodels0 != sub_nmodels1)
+    return sub_nmodels0 < sub_nmodels1 ? -1 : 1;
+  else
+  {
+    for (i = 0; i < sub_nmodels0; i++)
+    {
+      NcmModelID mid0  = ncm_mset_get_mid_array_pos (mset0, i);
+      NcmModelID mid1  = ncm_mset_get_mid_array_pos (mset1, i);
+      NcmModel *model0 = ncm_mset_peek_array_pos (mset0, i);
+      NcmModel *model1 = ncm_mset_peek_array_pos (mset1, i);
+
+      if (mid0 != mid1)
+        return mid0 < mid1 ? -1 : 1;
+      else if (!ncm_model_is_equal (model0, model1))
+        return ncm_model_len (model0) < ncm_model_len (model1) ? -1 : 1;
+    }
+  }
+
+  return 0;
+}
+
 /**
  * ncm_mset_get_id_by_type:
  * @model_type: FIXME
@@ -1815,7 +1852,7 @@ ncm_mset_param_set_ftype_from_fmap (NcmMSet *mset)
  * @mset: a #NcmMSet
  * @params: a #NcmVector
  *
- * Sets the models parameters using values from @params.
+ * Sets the models parameters using values from the #NcmVector @params.
  *
  */
 void
@@ -1873,6 +1910,30 @@ ncm_mset_param_get_vector (NcmMSet *mset, NcmVector *params)
   }
 }
 
+/**
+ * ncm_mset_param_set_mset:
+ * @mset_dest: a #NcmMSet
+ * @mset_src: a #NcmMSet
+ *
+ * Copy parameters from @mset_src to @mset_dest, both #NcmMSet must
+ * be compatible.
+ *
+ */
+void 
+ncm_mset_param_set_mset (NcmMSet *mset_dest, NcmMSet *mset_src)
+{
+  g_assert_cmpint (ncm_mset_cmp_all (mset_dest, mset_src), ==, 0);
+  {
+    const guint n     = ncm_mset_total_len (mset_dest);
+    NcmVector *params = ncm_vector_new (n);
+
+    ncm_mset_param_get_vector (mset_src, params);
+    ncm_mset_param_set_vector (mset_dest, params);
+
+    ncm_vector_free (params);
+  }
+}
+
 /**
  * ncm_mset_param_get_ftype:
  * @mset: a #NcmMSet
