@@ -32,7 +32,9 @@
 #include <numcosmo/math/ncm_model.h>
 #include <numcosmo/math/ncm_obj_array.h>
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <stdio.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_BEGIN_DECLS
 
@@ -174,6 +176,7 @@ void ncm_mset_set_pos (NcmMSet *mset, NcmModel *model, guint stackpos_id);
 gboolean ncm_mset_exists (NcmMSet *mset, NcmModel *model);
 gboolean ncm_mset_exists_pos (NcmMSet *mset, NcmModel *model, guint stackpos_id);
 gboolean ncm_mset_is_subset (NcmMSet *mset, NcmMSet *sub_mset);
+gint ncm_mset_cmp_all (NcmMSet *mset0, NcmMSet *mset1);
 
 gint ncm_mset_get_id_by_type (GType model_type);
 gint ncm_mset_get_id_by_ns (const gchar *ns);
@@ -214,6 +217,7 @@ void ncm_mset_param_set_all_but_mid_ftype (NcmMSet *mset, NcmModelID mid, NcmPar
 void ncm_mset_param_set_ftype_from_fmap (NcmMSet *mset);
 void ncm_mset_param_set_vector (NcmMSet *mset, NcmVector *params);
 void ncm_mset_param_get_vector (NcmMSet *mset, NcmVector *params);
+void ncm_mset_param_set_mset (NcmMSet *mset_dest, NcmMSet *mset_src);
 gdouble ncm_mset_param_get_scale (NcmMSet *mset, NcmModelID mid, guint pid);
 gdouble ncm_mset_param_get_lower_bound (NcmMSet *mset, NcmModelID mid, guint pid);
 gdouble ncm_mset_param_get_upper_bound (NcmMSet *mset, NcmModelID mid, guint pid);
