@@ -32,12 +32,6 @@
 #include <numcosmo/math/ncm_fit.h>
 #include <numcosmo/math/ncm_stats_vec.h>
 #include <numcosmo/math/ncm_stats_dist1d_epdf.h>
-#include <gsl/gsl_histogram.h>
-#include <gsl/gsl_eigen.h>
-#include <gsl/gsl_vector_complex.h>
-#ifdef NUMCOSMO_HAVE_CFITSIO
-#include <fitsio.h>
-#endif /* NUMCOSMO_HAVE_CFITSIO */
 
 G_BEGIN_DECLS
 
@@ -50,6 +44,7 @@ G_BEGIN_DECLS
 
 typedef struct _NcmMSetCatalogClass NcmMSetCatalogClass;
 typedef struct _NcmMSetCatalog NcmMSetCatalog;
+typedef struct _NcmMSetCatalogPrivate NcmMSetCatalogPrivate;
 
 struct _NcmMSetCatalogClass
 {
@@ -70,60 +65,48 @@ typedef enum _NcmMSetCatalogSync
 {
   NCM_MSET_CATALOG_SYNC_DISABLE,
   NCM_MSET_CATALOG_SYNC_AUTO,
-  NCM_MSET_CATALOG_SYNC_TIMED, /*< private >*/
-  NCM_MSET_CATALOG_SYNC_LEN,   /*< skip >*/
+  NCM_MSET_CATALOG_SYNC_TIMED, 
+  /* < private > */
+  NCM_MSET_CATALOG_SYNC_LEN, /*< skip >*/
 } NcmMSetCatalogSync;
 
+/**
+ * NcmMSetCatalogTrimType:
+ * @NCM_MSET_CATALOG_TRIM_TYPE_ESS: trim the catalog using the maximum ess criterium.
+ * @NCM_MSET_CATALOG_TRIM_TYPE_HEIDEL: trim the catalog using the Heidelberger and Welch’s convergence diagnostic.
+ * @NCM_MSET_CATALOG_TRIM_TYPE_ALL: trim the catalog using all tests above.
+ * 
+ * See ncm_mset_catalog_calc_max_ess_time() and ncm_mset_catalog_calc_heidel_diag().
+ * 
+ */
+typedef enum _NcmMSetCatalogTrimType
+{
+  NCM_MSET_CATALOG_TRIM_TYPE_ESS    = 1 << 0,
+  NCM_MSET_CATALOG_TRIM_TYPE_HEIDEL = 1 << 1,
+  NCM_MSET_CATALOG_TRIM_TYPE_ALL    = (1 << 2) - 1,
+} NcmMSetCatalogTrimType;
+
+/**
+ * NcmMSetCatalogTauMethod:
+ * @NCM_MSET_CATALOG_TAU_METHOD_ACOR: uses the autocorrelation to estimate $\tau$.
+ * @NCM_MSET_CATALOG_TAU_METHOD_AR_MODEL: uses an autoregressive model fitting to estimate $\tau$.
+ * 
+ * Method used to estimate the autocorrelation time $\tau$.
+ * 
+ */
+typedef enum _NcmMSetCatalogTauMethod
+{
+  NCM_MSET_CATALOG_TAU_METHOD_ACOR = 0,
+  NCM_MSET_CATALOG_TAU_METHOD_AR_MODEL, 
+  /* < private > */
+  NCM_MSET_CATALOG_TAU_METHOD_LEN, /*< skip >*/
+} NcmMSetCatalogTauMethod;
+
 struct _NcmMSetCatalog
 {
   /*< private >*/
   GObject parent_instance;
-  NcmMSet *mset;
-  guint nadd_vals;
-  GPtrArray *add_vals_names;
-  GPtrArray *add_vals_symbs;
-  NcmStatsVec *pstats;
-  NcmMSetCatalogSync smode;
-  gboolean readonly;
-  NcmRNG *rng;
-  gboolean weighted;
-  gboolean first_flush;
-  guint nchains;
-  GPtrArray *chain_pstats;
-  NcmStatsVec *mean_pstats;
-  NcmStatsVec *e_stats;
-  GPtrArray *e_mean_array;
-  GPtrArray *e_var_array;
-  NcmVector *chain_means;
-  NcmVector *chain_vars;
-  NcmMatrix *chain_cov;
-  NcmMatrix *chain_sM;
-  gsl_eigen_nonsymm_workspace *chain_sM_ws;
-  gsl_vector_complex *chain_sM_ev;
-  NcmVector *tau;
-  gchar *rng_inis;
-  gchar *rng_stat;
-  GTimer *sync_timer;
-  gdouble sync_interval;
-  gchar *file;
-  gchar *mset_file;
-  gchar *rtype_str;
-  GArray *porder;
-  NcmVector *quantile_ws;
-  gint first_id;
-  gint cur_id;
-  gint file_first_id;
-  gint file_cur_id;
-  glong burnin;
-#ifdef NUMCOSMO_HAVE_CFITSIO
-  fitsfile *fptr;
-#endif /* NUMCOSMO_HAVE_CFITSIO */
-  NcmVector *params_max;
-  NcmVector *params_min;
-  glong pdf_i;
-  gsl_histogram *h;
-  gsl_histogram_pdf *h_pdf;
-  gboolean constructed;
+  NcmMSetCatalogPrivate *priv;
 };
 
 GType ncm_mset_catalog_get_type (void) G_GNUC_CONST;
@@ -149,26 +132,44 @@ void ncm_mset_catalog_reset_stats (NcmMSetCatalog *mcat);
 void ncm_mset_catalog_reset (NcmMSetCatalog *mcat);
 void ncm_mset_catalog_erase_data (NcmMSetCatalog *mcat);
 
+void ncm_mset_catalog_set_m2lnp_var (NcmMSetCatalog *mcat, const gint p);
+gint ncm_mset_catalog_get_m2lnp_var (NcmMSetCatalog *mcat);
+
 const gchar *ncm_mset_catalog_peek_filename (NcmMSetCatalog *mcat);
 NcmRNG *ncm_mset_catalog_get_rng (NcmMSetCatalog *mcat);
+NcmRNG *ncm_mset_catalog_peek_rng (NcmMSetCatalog *mcat);
 
 gboolean ncm_mset_catalog_is_empty (NcmMSetCatalog *mcat);
 gdouble ncm_mset_catalog_largest_error (NcmMSetCatalog *mcat);
 guint ncm_mset_catalog_len (NcmMSetCatalog *mcat);
 guint ncm_mset_catalog_max_time (NcmMSetCatalog *mcat);
+guint ncm_mset_catalog_nchains (NcmMSetCatalog *mcat);
+guint ncm_mset_catalog_nadd_vals (NcmMSetCatalog *mcat);
+gboolean ncm_mset_catalog_weighted (NcmMSetCatalog *mcat);
+guint ncm_mset_catalog_get_row_from_time (NcmMSetCatalog *mcat, gint t);
+gint ncm_mset_catalog_get_first_id (NcmMSetCatalog *mcat);
+gint ncm_mset_catalog_get_cur_id (NcmMSetCatalog *mcat);
 
 void ncm_mset_catalog_set_burnin (NcmMSetCatalog *mcat, glong burnin);
 glong ncm_mset_catalog_get_burnin (NcmMSetCatalog *mcat);
 
+void ncm_mset_catalog_set_tau_method (NcmMSetCatalog *mcat, NcmMSetCatalogTauMethod tau_method);
+NcmMSetCatalogTauMethod ncm_mset_catalog_get_tau_method (NcmMSetCatalog *mcat);
+
 void ncm_mset_catalog_add_from_mset (NcmMSetCatalog *mcat, NcmMSet *mset, ...) G_GNUC_NULL_TERMINATED;
 void ncm_mset_catalog_add_from_mset_array (NcmMSetCatalog *mcat, NcmMSet *mset, gdouble *ax);
 void ncm_mset_catalog_add_from_vector (NcmMSetCatalog *mcat, NcmVector *vals);
+void ncm_mset_catalog_add_from_vector_array (NcmMSetCatalog *mcat, NcmVector *vals, gdouble *ax);
 void ncm_mset_catalog_log_current_stats (NcmMSetCatalog *mcat);
 void ncm_mset_catalog_log_current_chain_stats (NcmMSetCatalog *mcat);
 
 NcmMSet *ncm_mset_catalog_get_mset (NcmMSetCatalog *mcat);
+NcmMSet *ncm_mset_catalog_peek_mset (NcmMSetCatalog *mcat);
 const gchar *ncm_mset_catalog_get_run_type (NcmMSetCatalog *mcat);
 
+NcmStatsVec *ncm_mset_catalog_peek_pstats (NcmMSetCatalog *mcat);
+NcmStatsVec *ncm_mset_catalog_peek_e_mean_stats (NcmMSetCatalog *mcat);
+NcmStatsVec *ncm_mset_catalog_peek_chain_pstats (NcmMSetCatalog *mcat, const guint i);
 NcmVector *ncm_mset_catalog_peek_row (NcmMSetCatalog *mcat, guint i);
 NcmVector *ncm_mset_catalog_peek_current_row (NcmMSetCatalog *mcat);
 NcmVector *ncm_mset_catalog_peek_current_e_mean (NcmMSetCatalog *mcat);
@@ -176,21 +177,25 @@ NcmVector *ncm_mset_catalog_peek_current_e_var (NcmMSetCatalog *mcat);
 NcmVector *ncm_mset_catalog_peek_e_mean_t (NcmMSetCatalog *mcat, guint t);
 NcmVector *ncm_mset_catalog_peek_e_var_t (NcmMSetCatalog *mcat, guint t);
 
+gdouble ncm_mset_catalog_get_post_lnnorm (NcmMSetCatalog *mcat);
+gdouble ncm_mset_catalog_get_post_lnvol (NcmMSetCatalog *mcat, const gdouble level, gdouble *glnvol);
+
 void ncm_mset_catalog_get_mean (NcmMSetCatalog *mcat, NcmVector  **mean);
 void ncm_mset_catalog_get_covar (NcmMSetCatalog *mcat, NcmMatrix **cov);
 void ncm_mset_catalog_get_full_covar (NcmMSetCatalog *mcat, NcmMatrix **cov);
 void ncm_mset_catalog_log_full_covar (NcmMSetCatalog *mcat);
 
-void ncm_mset_catalog_estimate_autocorrelation_tau (NcmMSetCatalog *mcat);
+void ncm_mset_catalog_estimate_autocorrelation_tau (NcmMSetCatalog *mcat, gboolean force_single_chain);
 NcmVector *ncm_mset_catalog_peek_autocorrelation_tau (NcmMSetCatalog *mcat);
 gdouble ncm_mset_catalog_get_param_shrink_factor (NcmMSetCatalog *mcat, guint p);
 gdouble ncm_mset_catalog_get_shrink_factor (NcmMSetCatalog *mcat);
 
 void ncm_mset_catalog_param_pdf (NcmMSetCatalog *mcat, guint i);
-gdouble ncm_mset_catalog_param_pdf_pvalue (NcmMSetCatalog *mcat, gdouble pval, gboolean both);
+gdouble ncm_mset_catalog_param_pdf_pvalue (NcmMSetCatalog *mcat, gdouble pvalue, gboolean both);
 
 NcmMatrix *ncm_mset_catalog_calc_ci_direct (NcmMSetCatalog *mcat, NcmMSetFunc *func, NcmVector *x_v, GArray *p_val);
 NcmMatrix *ncm_mset_catalog_calc_ci_interp (NcmMSetCatalog *mcat, NcmMSetFunc *func, NcmVector *x_v, GArray *p_val, guint nodes, NcmFitRunMsgs mtype);
+NcmMatrix *ncm_mset_catalog_calc_pvalue (NcmMSetCatalog *mcat, NcmMSetFunc *func, NcmVector *x_v, GArray *lim, guint nodes, NcmFitRunMsgs mtype);
 NcmStatsDist1d *ncm_mset_catalog_calc_distrib (NcmMSetCatalog *mcat, NcmMSetFunc *func, NcmFitRunMsgs mtype);
 NcmStatsDist1d *ncm_mset_catalog_calc_param_distrib (NcmMSetCatalog *mcat, const NcmMSetPIndex *pi, NcmFitRunMsgs mtype);
 NcmStatsDist1d *ncm_mset_catalog_calc_add_param_distrib (NcmMSetCatalog *mcat, guint add_param, NcmFitRunMsgs mtype);
@@ -198,10 +203,22 @@ NcmStatsDist1d *ncm_mset_catalog_calc_add_param_distrib (NcmMSetCatalog *mcat, g
 void ncm_mset_catalog_calc_param_ensemble_evol (NcmMSetCatalog *mcat, const NcmMSetPIndex *pi, guint nsteps, NcmFitRunMsgs mtype, NcmVector **pval, NcmMatrix **t_evol);
 void ncm_mset_catalog_calc_add_param_ensemble_evol (NcmMSetCatalog *mcat, guint add_param, guint nsteps, NcmFitRunMsgs mtype, NcmVector **pval, NcmMatrix **t_evol);
 
+void ncm_mset_catalog_trim (NcmMSetCatalog *mcat, const guint tc);
+void ncm_mset_catalog_remove_last_ensemble (NcmMSetCatalog *mcat);
+
+guint ncm_mset_catalog_calc_max_ess_time (NcmMSetCatalog *mcat, const guint ntests, gdouble *max_ess, NcmFitRunMsgs mtype);
+guint ncm_mset_catalog_calc_heidel_diag (NcmMSetCatalog *mcat, const guint ntests, const gdouble pvalue, NcmFitRunMsgs mtype);
+
+void ncm_mset_catalog_trim_by_type (NcmMSetCatalog *mcat, guint ntests, NcmMSetCatalogTrimType trim_type, NcmFitRunMsgs mtype);
+
+guint ncm_mset_catalog_max_ess_time_by_chain (NcmMSetCatalog *mcat, const guint ntests, gdouble *max_ess, NcmFitRunMsgs mtype);
+guint ncm_mset_catalog_heidel_diag_by_chain (NcmMSetCatalog *mcat, const guint ntests, const gdouble pvalue, gdouble *wp_pvalue, NcmFitRunMsgs mtype);
+
 #define NCM_MSET_CATALOG_EXTNAME "NcmMSetCatalog:DATA"
 #define NCM_MSET_CATALOG_M2LNL_COLNAME "NcmFit:m2lnL"
 #define NCM_MSET_CATALOG_M2LNL_SYMBOL "-2\\ln(L)"
 #define NCM_MSET_CATALOG_FIRST_ID_LABEL "FIRST_ID"
+#define NCM_MSET_CATALOG_M2LNP_ID_LABEL "M2LNP_ID"
 #define NCM_MSET_CATALOG_RNG_ALGO_LABEL "RNG_ALGO"
 #define NCM_MSET_CATALOG_RNG_SEED_LABEL "RNG_SEED"
 #define NCM_MSET_CATALOG_RNG_STAT_LABEL "RNG_STAT"
