@@ -43,23 +43,6 @@ G_BEGIN_DECLS
 typedef struct _NcmSpline2dClass NcmSpline2dClass;
 typedef struct _NcmSpline2d NcmSpline2d;
 
-struct _NcmSpline2d
-{
-  /*< private >*/
-  GObject parent_instance;
-  gboolean empty;
-  gboolean init;
-  gboolean to_init;
-  NcmSpline *s;
-  NcmVector *xv;
-  NcmVector *yv;
-  NcmMatrix *zm;
-  gsl_interp_accel *acc_x;
-  gsl_interp_accel *acc_y;
-  gboolean use_acc;
-  gboolean no_stride;
-};
-
 struct _NcmSpline2dClass
 {
   /*< private >*/
@@ -80,6 +63,23 @@ struct _NcmSpline2dClass
   NcmSpline *(*int_dy_spline) (NcmSpline2d *s2d, gdouble yl, gdouble yu);
 };
 
+struct _NcmSpline2d
+{
+  /*< private >*/
+  GObject parent_instance;
+  gboolean empty;
+  gboolean init;
+  gboolean to_init;
+  NcmSpline *s;
+  NcmVector *xv;
+  NcmVector *yv;
+  NcmMatrix *zm;
+  gsl_interp_accel *acc_x;
+  gsl_interp_accel *acc_y;
+  gboolean use_acc;
+  gboolean no_stride;
+};
+
 GType ncm_spline2d_get_type (void) G_GNUC_CONST;
 
 void ncm_spline2d_set (NcmSpline2d *s2d, NcmVector *xv, NcmVector *yv, NcmMatrix *zm, gboolean init);
@@ -108,13 +108,18 @@ gdouble ncm_spline2d_integ_dy_spline_val (NcmSpline2d *s2d, gdouble x, gdouble y
 gdouble ncm_spline2d_integ_dxdy_spline_x (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdouble yl, gdouble yu);
 gdouble ncm_spline2d_integ_dxdy_spline_y (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdouble yl, gdouble yu);
 
+G_INLINE_FUNC gdouble ncm_spline2d_deriv_dzdx (NcmSpline2d *s2d, gdouble x, gdouble y);
+G_INLINE_FUNC gdouble ncm_spline2d_deriv_dzdy (NcmSpline2d *s2d, gdouble x, gdouble y);
+G_INLINE_FUNC gdouble ncm_spline2d_deriv_d2zdxy (NcmSpline2d *s2d, gdouble x, gdouble y);
+G_INLINE_FUNC gdouble ncm_spline2d_deriv_d2zdx2 (NcmSpline2d *s2d, gdouble x, gdouble y);
+G_INLINE_FUNC gdouble ncm_spline2d_deriv_d2zdy2 (NcmSpline2d *s2d, gdouble x, gdouble y);
+
 G_INLINE_FUNC gdouble ncm_spline2dim_integ_total (NcmSpline2d *s2d);
 
 G_END_DECLS
 
 #endif /* _NCM_SPLINE2D_H_ */
 
-
 #ifndef _NCM_SPLINE2D_INLINE_H_
 #define _NCM_SPLINE2D_INLINE_H_
 #ifdef NUMCOSMO_HAVE_INLINE
@@ -140,6 +145,46 @@ ncm_spline2dim_integ_total (NcmSpline2d *s2d)
 	                                );
 }
 
+G_INLINE_FUNC gdouble 
+ncm_spline2d_deriv_dzdx (NcmSpline2d *s2d, gdouble x, gdouble y)
+{
+  if (!s2d->init)
+    ncm_spline2d_prepare (s2d);
+  return NCM_SPLINE2D_GET_CLASS (s2d)->dzdx (s2d, x, y);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_spline2d_deriv_dzdy (NcmSpline2d *s2d, gdouble x, gdouble y)
+{
+  if (!s2d->init)
+    ncm_spline2d_prepare (s2d);
+  return NCM_SPLINE2D_GET_CLASS (s2d)->dzdy (s2d, x, y);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_spline2d_deriv_d2zdxy (NcmSpline2d *s2d, gdouble x, gdouble y)
+{
+  if (!s2d->init)
+    ncm_spline2d_prepare (s2d);
+  return NCM_SPLINE2D_GET_CLASS (s2d)->d2zdxy (s2d, x, y);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_spline2d_deriv_d2zdx2 (NcmSpline2d *s2d, gdouble x, gdouble y)
+{
+  if (!s2d->init)
+    ncm_spline2d_prepare (s2d);
+  return NCM_SPLINE2D_GET_CLASS (s2d)->d2zdx2 (s2d, x, y);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_spline2d_deriv_d2zdy2 (NcmSpline2d *s2d, gdouble x, gdouble y)
+{
+  if (!s2d->init)
+    ncm_spline2d_prepare (s2d);
+  return NCM_SPLINE2D_GET_CLASS (s2d)->d2zdy2 (s2d, x, y);
+}
+
 G_END_DECLS
 
 #endif /* NUMCOSMO_HAVE_INLINE */
