@@ -25,12 +25,15 @@
 #ifndef _NCM_VECTOR_H_
 #define _NCM_VECTOR_H_
 
-#include <string.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
+
+#ifndef NUMCOSMO_GIR_SCAN
+#include <string.h>
 #include <gsl/gsl_vector.h>
 #include <sundials/sundials_nvector.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_BEGIN_DECLS
 
@@ -103,16 +106,21 @@ const NcmVector *ncm_vector_const_new_variant (GVariant *var);
 const NcmVector *ncm_vector_const_new_data (const gdouble *d, const gsize size, const gsize stride);
 
 NcmVector *ncm_vector_get_subvector (NcmVector *cv, const gsize k, const gsize size);
+NcmVector *ncm_vector_get_subvector_stride (NcmVector *cv, const gsize k, const gsize size, const gsize stride);
 GVariant *ncm_vector_get_variant (const NcmVector *v);
 GVariant *ncm_vector_peek_variant (const NcmVector *v);
 
-void ncm_vector_log_vals (const NcmVector *v, const gchar *prestr, const gchar *format);
+void ncm_vector_log_vals (const NcmVector *v, const gchar *prestr, const gchar *format, gboolean cr);
 void ncm_vector_log_vals_avpb (const NcmVector *v, const gchar *prestr, const gchar *format, const gdouble a, const gdouble b);
 void ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *format, NcmVectorCompFunc f, gpointer user_data);
 
 void ncm_vector_set_from_variant (NcmVector *cv, GVariant *var);
 
 gdouble ncm_vector_dnrm2 (const NcmVector *cv);
+void ncm_vector_axpy (NcmVector *cv1, const gdouble alpha, const NcmVector *cv2);
+void ncm_vector_cmp (NcmVector *cv1, const NcmVector *cv2);
+void ncm_vector_sub_round_off (NcmVector *cv1, const NcmVector *cv2);
+void ncm_vector_reciprocal (NcmVector *cv);
 
 G_INLINE_FUNC gdouble ncm_vector_sum_cpts (const NcmVector *cv);
 G_INLINE_FUNC const NcmVector *ncm_vector_const_new_gsl (const gsl_vector *v);
@@ -130,9 +138,11 @@ G_INLINE_FUNC void ncm_vector_fast_subfrom (NcmVector *cv, const guint i, const
 G_INLINE_FUNC void ncm_vector_mulby (NcmVector *cv, const guint i, const gdouble val);
 G_INLINE_FUNC void ncm_vector_fast_mulby (NcmVector *cv, const guint i, const gdouble val);
 G_INLINE_FUNC void ncm_vector_set_all (NcmVector *cv, const gdouble val);
-G_INLINE_FUNC void ncm_vector_set_array (NcmVector *cv, const gdouble *array);
+G_INLINE_FUNC void ncm_vector_set_data (NcmVector *cv, const gdouble *array, guint size);
+G_INLINE_FUNC void ncm_vector_set_array (NcmVector *cv, GArray *array);
 G_INLINE_FUNC void ncm_vector_scale (NcmVector *cv, const gdouble val);
 G_INLINE_FUNC void ncm_vector_add_constant (NcmVector *cv, const gdouble val);
+G_INLINE_FUNC void ncm_vector_mul (NcmVector *cv1, const NcmVector *cv2);
 G_INLINE_FUNC void ncm_vector_div (NcmVector *cv1, const NcmVector *cv2);
 G_INLINE_FUNC void ncm_vector_add (NcmVector *cv1, const NcmVector *cv2);
 G_INLINE_FUNC void ncm_vector_sub (NcmVector *cv1, const NcmVector *cv2);
@@ -149,7 +159,13 @@ G_INLINE_FUNC const gsl_vector *ncm_vector_const_gsl (const NcmVector *cv);
 G_INLINE_FUNC guint ncm_vector_len (const NcmVector *cv);
 G_INLINE_FUNC guint ncm_vector_stride (const NcmVector *cv);
 
-void ncm_vector_get_minmax (const NcmVector *cv, gdouble *min, gdouble *max);
+G_INLINE_FUNC gdouble ncm_vector_get_max (const NcmVector *cv);
+G_INLINE_FUNC gdouble ncm_vector_get_min (const NcmVector *cv);
+G_INLINE_FUNC gsize ncm_vector_get_max_index (const NcmVector *cv);
+G_INLINE_FUNC gsize ncm_vector_get_min_index (const NcmVector *cv);
+
+G_INLINE_FUNC void ncm_vector_get_minmax (const NcmVector *cv, gdouble *min, gdouble *max);
+
 void ncm_vector_get_absminmax (const NcmVector *cv, gdouble *absmin, gdouble *absmax);
 
 NcmVector *ncm_vector_dup (const NcmVector *cv);
@@ -170,7 +186,7 @@ G_END_DECLS
 
 G_BEGIN_DECLS
 
-G_INLINE_FUNC gdouble 
+G_INLINE_FUNC gdouble
 ncm_vector_sum_cpts (const NcmVector *cv)
 {
   guint i;
@@ -271,14 +287,29 @@ ncm_vector_set_all (NcmVector *cv, const gdouble val)
 }
 
 G_INLINE_FUNC void
-ncm_vector_set_array (NcmVector *cv, const gdouble *array)
+ncm_vector_set_data (NcmVector *cv, const gdouble *array, guint size)
 {
   register guint i;
-  const guint size = ncm_vector_len (cv);
+  const guint vsize = ncm_vector_len (cv);
+
+  g_assert_cmpuint (vsize, ==, size);
+  
   for (i = 0; i < size; i++)
     ncm_vector_set (cv, i, array[i]);
 }
 
+G_INLINE_FUNC void
+ncm_vector_set_array (NcmVector *cv, GArray *array)
+{
+  register guint i;
+  const guint vsize = ncm_vector_len (cv);
+
+  g_assert_cmpuint (vsize, ==, array->len);
+  
+  for (i = 0; i < vsize; i++)
+    ncm_vector_set (cv, i, g_array_index (array, gdouble, i));
+}
+
 G_INLINE_FUNC void
 ncm_vector_scale (NcmVector *cv, const gdouble val)
 {
@@ -291,10 +322,16 @@ ncm_vector_add_constant (NcmVector *cv, const gdouble val)
   gsl_vector_add_constant (ncm_vector_gsl (cv), val);
 }
 
+G_INLINE_FUNC void
+ncm_vector_mul (NcmVector *cv1, const NcmVector *cv2)
+{
+  gsl_vector_mul (ncm_vector_gsl (cv1), ncm_vector_const_gsl (cv2));
+}
+
 G_INLINE_FUNC void
 ncm_vector_div (NcmVector *cv1, const NcmVector *cv2)
 {
-  gsl_vector_div (ncm_vector_gsl(cv1), ncm_vector_const_gsl(cv2));
+  gsl_vector_div (ncm_vector_gsl (cv1), ncm_vector_const_gsl (cv2));
 }
 
 G_INLINE_FUNC void
@@ -325,6 +362,9 @@ ncm_vector_memcpy (NcmVector *cv1, const NcmVector *cv2)
 G_INLINE_FUNC void
 ncm_vector_memcpy2 (NcmVector *cv1, const NcmVector *cv2, const guint cv1_start, const guint cv2_start, const guint size)
 {
+  g_assert_cmpuint (ncm_vector_len (cv1), >=, size + cv1_start);
+  g_assert_cmpuint (ncm_vector_len (cv2), >=, size + cv2_start);
+  
   memcpy (ncm_vector_ptr (cv1, cv1_start), ncm_vector_const_ptr (cv2, cv2_start), sizeof (gdouble) * size);
 }
 
@@ -380,6 +420,36 @@ ncm_vector_stride (const NcmVector *cv)
   return cv->vv.vector.stride;
 }
 
+G_INLINE_FUNC gdouble 
+ncm_vector_get_max (const NcmVector *cv)
+{
+  return gsl_vector_max (ncm_vector_const_gsl (cv));
+}
+
+G_INLINE_FUNC gdouble 
+ncm_vector_get_min (const NcmVector *cv)
+{
+  return gsl_vector_min (ncm_vector_const_gsl (cv));
+}
+
+G_INLINE_FUNC gsize 
+ncm_vector_get_max_index (const NcmVector *cv)
+{
+  return gsl_vector_max_index (ncm_vector_const_gsl (cv));
+}
+
+G_INLINE_FUNC gsize 
+ncm_vector_get_min_index (const NcmVector *cv)
+{
+  return gsl_vector_min_index (ncm_vector_const_gsl (cv));
+}
+
+G_INLINE_FUNC void 
+ncm_vector_get_minmax (const NcmVector *cv, gdouble *min, gdouble *max)
+{
+  gsl_vector_minmax (ncm_vector_const_gsl (cv), min, max);
+}
+
 G_END_DECLS
 
 #endif /* NUMCOSMO_HAVE_INLINE */
