@@ -30,7 +30,9 @@
  *
  * Provides the nonlinear matter power spectrum using Halofit model [Smith et al (2003)][XSmith2003]
  * and [Takahashi et al. (2012)][XTakahashi2012] FIXME.
- *
+ * 
+ * For PKEqual see [Casarini et al. (2009)][XCasarini2009] and [Casarini et al. (2016)][XCasarini2016].
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,14 +44,18 @@
 #include "nc_powspec_mnl_halofit.h"
 
 #include "model/nc_hicosmo_de.h"
+#include "model/nc_hicosmo_de_cpl.h"
+#include "nc_distance.h"
 
 #include "math/integral.h"
 #include "math/memory_pool.h"
 #include "math/ncm_spline_cubic_notaknot.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_math.h>
 #include <gsl/gsl_roots.h>
 #include <gsl/gsl_sf_exp.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 struct _NcPowspecMNLHaloFitPrivate
 {
@@ -65,8 +71,13 @@ struct _NcPowspecMNLHaloFitPrivate
 	gdouble f1;
 	gdouble f2;
 	gdouble f3;
-	gsl_root_fdfsolver *linear_scale_solver;
-  gsl_root_fsolver *znl_solver;
+	gdouble mnu_corr_halo;
+	gdouble fnu;
+	gsl_root_fdfsolver* linear_scale_solver;
+	gsl_root_fsolver* znl_solver;
+  gboolean pkequal;
+  NcHICosmo *cpl;
+  NcDistance *cpl_dist;
 };
 
 enum
@@ -75,6 +86,7 @@ enum
 	PROP_PSML,
 	PROP_ZMAXNL,
 	PROP_RELTOL,
+  PROP_PKEQUAL,
 };
 
 G_DEFINE_TYPE (NcPowspecMNLHaloFit, nc_powspec_mnl_halofit, NC_TYPE_POWSPEC_MNL);
@@ -90,12 +102,16 @@ nc_powspec_mnl_halofit_init (NcPowspecMNLHaloFit* pshf)
 	pshf->neff       = NULL;
 	pshf->Cur        = NULL;
 	pshf->psml_gauss = NULL;
-	pshf->priv       = G_TYPE_INSTANCE_GET_PRIVATE (pshf, NC_TYPE_POWSPEC_MNL_HALOFIT, NcPowspecMNLHaloFitPrivate);
+
+	pshf->priv = G_TYPE_INSTANCE_GET_PRIVATE (pshf, NC_TYPE_POWSPEC_MNL_HALOFIT, NcPowspecMNLHaloFitPrivate);
 
 	pshf->priv->linear_scale_solver = gsl_root_fdfsolver_alloc (gsl_root_fdfsolver_steffenson);
 	pshf->priv->znl_solver          = gsl_root_fsolver_alloc (gsl_root_fsolver_brent);
 
-	pshf->priv->z = HUGE_VAL;
+	pshf->priv->z           = HUGE_VAL;
+  pshf->priv->pkequal     = FALSE;
+  pshf->priv->cpl      = NULL;
+  pshf->priv->cpl_dist = NULL;
 }
 
 static void
@@ -107,7 +123,7 @@ _nc_powspec_mnl_halofit_set_property (GObject* object, guint prop_id, const GVal
   switch (prop_id)
   {
     case PROP_PSML:
-      pshf->psml       = g_value_dup_object (value);
+      pshf->psml = g_value_dup_object (value);
       pshf->psml_gauss = ncm_powspec_filter_new (NCM_POWSPEC (pshf->psml), NCM_POWSPEC_FILTER_TYPE_GAUSS);
       break;
     case PROP_ZMAXNL:
@@ -116,6 +132,9 @@ _nc_powspec_mnl_halofit_set_property (GObject* object, guint prop_id, const GVal
     case PROP_RELTOL:
       pshf->reltol = g_value_get_double (value);
       break;
+    case PROP_PKEQUAL:
+      nc_powspec_mnl_halofit_pkequal (pshf, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -128,8 +147,8 @@ _nc_powspec_mnl_halofit_get_property (GObject* object, guint prop_id, GValue* va
 	NcPowspecMNLHaloFit* pshf = NC_POWSPEC_MNL_HALOFIT (object);
 	g_return_if_fail (NC_IS_POWSPEC_MNL_HALOFIT (object));
 
-  switch (prop_id)
-  {
+	switch (prop_id)
+	{
     case PROP_PSML:
       g_value_set_object (value, pshf->psml);
       break;
@@ -139,6 +158,9 @@ _nc_powspec_mnl_halofit_get_property (GObject* object, guint prop_id, GValue* va
     case PROP_RELTOL:
       g_value_set_double (value, pshf->reltol);
       break;
+    case PROP_PKEQUAL:
+      g_value_set_boolean (value, pshf->priv->pkequal);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -176,6 +198,9 @@ _nc_powspec_mnl_halofit_dispose (GObject* object)
 
 	ncm_powspec_filter_clear (&pshf->psml_gauss);
 
+  nc_hicosmo_clear (&pshf->priv->cpl);
+  nc_distance_clear (&pshf->priv->cpl_dist);
+
 	/* Chain up : end */
 	G_OBJECT_CLASS (nc_powspec_mnl_halofit_parent_class)->dispose (object);
 }
@@ -192,9 +217,9 @@ _nc_powspec_mnl_halofit_finalize (GObject* object)
 	G_OBJECT_CLASS (nc_powspec_mnl_halofit_parent_class)->finalize (object);
 }
 
-static void _nc_powspec_mnl_halofit_prepare (NcmPowspec* powspec, NcmModel *model);
-static gdouble _nc_powspec_mnl_halofit_eval (NcmPowspec* powspec, NcmModel *model, const gdouble z, const gdouble k);
-static void _nc_powspec_mnl_halofit_eval_vec (NcmPowspec* powspec, NcmModel *model, const gdouble z, NcmVector* k, NcmVector* Pk);
+static void _nc_powspec_mnl_halofit_prepare (NcmPowspec* powspec, NcmModel* model);
+static gdouble _nc_powspec_mnl_halofit_eval (NcmPowspec* powspec, NcmModel* model, const gdouble z, const gdouble k);
+static void _nc_powspec_mnl_halofit_eval_vec (NcmPowspec* powspec, NcmModel* model, const gdouble z, NcmVector* k, NcmVector* Pk);
 static void _nc_powspec_mnl_halofit_get_nknots (NcmPowspec* powspec, guint* Nz, guint* Nk);
 
 static void
@@ -206,9 +231,9 @@ nc_powspec_mnl_halofit_class_init (NcPowspecMNLHaloFitClass* klass)
 	object_class->set_property = &_nc_powspec_mnl_halofit_set_property;
 	object_class->get_property = &_nc_powspec_mnl_halofit_get_property;
 
-	object_class->constructed  = &_nc_powspec_mnl_halofit_constructed;
-	object_class->dispose      = &_nc_powspec_mnl_halofit_dispose;
-	object_class->finalize     = &_nc_powspec_mnl_halofit_finalize;
+	object_class->constructed = &_nc_powspec_mnl_halofit_constructed;
+	object_class->dispose = &_nc_powspec_mnl_halofit_dispose;
+	object_class->finalize = &_nc_powspec_mnl_halofit_finalize;
 
 	g_type_class_add_private (klass, sizeof (NcPowspecMNLHaloFitPrivate));
 
@@ -234,10 +259,17 @@ nc_powspec_mnl_halofit_class_init (NcPowspecMNLHaloFitClass* klass)
 	                                                      GSL_DBL_EPSILON, 1.0, 1e-3,
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
-
-	powspec_class->prepare    = &_nc_powspec_mnl_halofit_prepare;
-	powspec_class->eval       = &_nc_powspec_mnl_halofit_eval;
-	powspec_class->eval_vec   = &_nc_powspec_mnl_halofit_eval_vec;
+  g_object_class_install_property (object_class,
+                                   PROP_PKEQUAL,
+                                   g_param_spec_boolean ("use-pkequal",
+                                                         NULL,
+                                                         "Whether to use PKEqual",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
+	powspec_class->prepare = &_nc_powspec_mnl_halofit_prepare;
+	powspec_class->eval = &_nc_powspec_mnl_halofit_eval;
+	powspec_class->eval_vec = &_nc_powspec_mnl_halofit_eval_vec;
 	powspec_class->get_nknots = &_nc_powspec_mnl_halofit_get_nknots;
 }
 
@@ -285,15 +317,15 @@ _nc_powspec_mnl_halofit_var_moment (NcPowspecML* ps, NcHICosmo* cosmo, const gdo
 typedef struct _var_params
 {
 	NcPowspecMNLHaloFit* pshf;
-	NcHICosmo *cosmo;
+	NcHICosmo* cosmo;
 	const gdouble z;
-  const gdouble R_min;
+	const gdouble R_min;
 } var_params;
 
 static gdouble
 _nc_powspec_mnl_halofit_varm1 (gdouble lnR, gpointer params)
 {
-	var_params *vps = (var_params *)params;
+	var_params* vps = (var_params*)params;
 	/*
   const gdouble R = gsl_sf_exp (lnR);
   const gdouble sigma2_0 = _nc_powspec_mnl_halofit_var_moment (vps->pshf->psml, vps->cosmo, R, vps->z, 0);
@@ -330,8 +362,8 @@ _nc_powspec_mnl_halofit_varm1_fdf (gdouble lnR, gpointer params, gdouble* varm1,
 
 typedef struct _root_params
 {
-	NcPowspecML *ps;
-	NcHICosmo *cosmo;
+	NcPowspecML* ps;
+	NcHICosmo* cosmo;
 } root_params;
 
 static gdouble
@@ -340,18 +372,18 @@ _nc_powspec_mnl_halofit_linear_scale (NcPowspecMNLHaloFit* pshf, NcHICosmo* cosm
 	gint status;
 	gint iter = 0, max_iter = 20000;
 
-	gdouble lnR0          = 0.0;
-	gdouble lnR           = (-z / 2.0 < NC_POWSPEC_MNL_HALOFIT_LOGRMIN) ? NC_POWSPEC_MNL_HALOFIT_LOGRMIN : -z / 2.0 + NCM_DEFAULT_PRECISION;
-	const gdouble reltol  = pshf->reltol / 10.0;
-	gdouble res           = 0.0;
+	gdouble lnR0 = 0.0;
+	gdouble lnR = (-z / 2.0 < NC_POWSPEC_MNL_HALOFIT_LOGRMIN) ? NC_POWSPEC_MNL_HALOFIT_LOGRMIN : -z / 2.0 + NCM_DEFAULT_PRECISION;
+	const gdouble reltol = pshf->reltol / 10.0;
+	gdouble res = 0.0;
 
 	gsl_function_fdf FDF;
 
-	var_params vps = { pshf, cosmo, z, 0.0};
+	var_params vps = { pshf, cosmo, z, 0.0 };
 
-	FDF.f      = &_nc_powspec_mnl_halofit_varm1;
-	FDF.df     = &_nc_powspec_mnl_halofit_varm1_deriv;
-	FDF.fdf    = &_nc_powspec_mnl_halofit_varm1_fdf;
+	FDF.f = &_nc_powspec_mnl_halofit_varm1;
+	FDF.df = &_nc_powspec_mnl_halofit_varm1_deriv;
+	FDF.fdf = &_nc_powspec_mnl_halofit_varm1_fdf;
 	FDF.params = &vps;
 
 	gsl_root_fdfsolver_set (pshf->priv->linear_scale_solver, &FDF, lnR);
@@ -362,18 +394,56 @@ _nc_powspec_mnl_halofit_linear_scale (NcPowspecMNLHaloFit* pshf, NcHICosmo* cosm
 		status = gsl_root_fdfsolver_iterate (pshf->priv->linear_scale_solver);
 
 		lnR0 = lnR;
-		lnR  = gsl_root_fdfsolver_root (pshf->priv->linear_scale_solver);
+		lnR = gsl_root_fdfsolver_root (pshf->priv->linear_scale_solver);
 
-		res  = gsl_expm1 (lnR0 - lnR);
+		res = gsl_expm1 (lnR0 - lnR);
 
 		status = gsl_root_test_residual (res, reltol); //Compares R vs R0 !
 
 	} while (status == GSL_CONTINUE && iter < max_iter);
 
-	if (iter >= max_iter)
-		g_warning ("_nc_powspec_mnl_halofit_linear_scale: maximum number of iteration reached (%u), giving up.", max_iter);
+	res = exp (lnR); // Now res is the result
 
-	res = exp (lnR);
+	if (iter >= max_iter)
+		g_warning ("_nc_powspec_mnl_halofit_linear_scale: maximum number of iteration reached (%u), non-linear scale found R(z=%.3f).", max_iter, z);
+
+	if (!(gsl_finite (res)))
+		g_warning ("_nc_powspec_mnl_halofit_linear_scale: non-linear scale found R(z=%.3f) not finite.", z);
+
+	if ((iter >= max_iter) || !(gsl_finite (res)))
+	{
+		g_message ("_nc_powspec_mnl_halofit_linear_scale: running the bracketing solver...");
+
+		iter = 0;
+
+		gsl_root_fsolver* s = gsl_root_fsolver_alloc (gsl_root_fsolver_brent);
+
+		gsl_function F;
+		F.function = &_nc_powspec_mnl_halofit_varm1;
+		F.params = &vps;
+
+		gdouble lnRlo = -z / 2. - 10.;
+		gdouble lnRup = -z / 2. + 10.;
+
+		gsl_root_fsolver_set (s, &F, lnRlo, lnRup);
+
+		do
+		{
+			iter++;
+			status = gsl_root_fsolver_iterate (s);
+
+			lnR = gsl_root_fsolver_root (s);
+			lnRlo = gsl_root_fsolver_x_lower (s);
+			lnRup = gsl_root_fsolver_x_upper (s);
+
+			status = gsl_root_test_interval (lnRlo, lnRup, reltol, 0.0); // dlnR = dR/R, so test with abstol
+
+		} while (status == GSL_CONTINUE && iter < max_iter);
+
+		gsl_root_fsolver_free (s);
+
+		res = exp (lnR);
+	}
 
 	return res;
 }
@@ -381,7 +451,7 @@ _nc_powspec_mnl_halofit_linear_scale (NcPowspecMNLHaloFit* pshf, NcHICosmo* cosm
 static gdouble
 _nc_powspec_mnl_halofit_linear_scale_z (gdouble z, gpointer params)
 {
-	var_params *vps = (var_params *)params;
+	var_params* vps = (var_params*)params;
 
 	return _nc_powspec_mnl_halofit_linear_scale (vps->pshf, vps->cosmo, z);
 }
@@ -389,15 +459,15 @@ _nc_powspec_mnl_halofit_linear_scale_z (gdouble z, gpointer params)
 static gdouble
 _nc_powspec_mnl_halofit_linear_scale_z_znl (gdouble z, gpointer params)
 {
-	var_params *vps = (var_params *)params;
+	var_params* vps = (var_params*)params;
 
 	return _nc_powspec_mnl_halofit_linear_scale (vps->pshf, vps->cosmo, z) - vps->R_min;
 }
 
 static void
-_nc_powspec_mnl_halofit_prepare_nl (NcPowspecMNLHaloFit* pshf, NcmModel *model)
+_nc_powspec_mnl_halofit_prepare_nl (NcPowspecMNLHaloFit* pshf, NcmModel* model)
 {
-	NcHICosmo *cosmo = NC_HICOSMO (model);
+	NcHICosmo* cosmo = NC_HICOSMO (model);
 	guint i;
 
 	pshf->priv->z = HUGE_VAL;
@@ -415,76 +485,76 @@ _nc_powspec_mnl_halofit_prepare_nl (NcPowspecMNLHaloFit* pshf, NcmModel *model)
 	ncm_powspec_filter_prepare_if_needed (pshf->psml_gauss, model);
 
 	{
-    const gdouble R_min = ncm_powspec_filter_get_r_min (pshf->psml_gauss);
-		var_params vps = { pshf, cosmo, 0.0, R_min};
+		const gdouble R_min = ncm_powspec_filter_get_r_min (pshf->psml_gauss);
+		var_params vps = { pshf, cosmo, 0.0, R_min };
 
-    gsl_function Fznl;
+		gsl_function Fznl;
 
-    Fznl.function = &_nc_powspec_mnl_halofit_linear_scale_z_znl;
-    Fznl.params   = &vps;
+		Fznl.function = &_nc_powspec_mnl_halofit_linear_scale_z_znl;
+		Fznl.params = &vps;
 
-    if (_nc_powspec_mnl_halofit_linear_scale_z (pshf->zmaxnl, Fznl.params) > R_min)
-      pshf->znl = pshf->zmaxnl;
-    else
-    {
-      const gdouble step = 1.0;
-      gdouble z0     = 0.0;
-      gdouble z1     = z0 + step;
-      gdouble R0     = _nc_powspec_mnl_halofit_linear_scale_z (z0, Fznl.params);
-      gdouble R1     = _nc_powspec_mnl_halofit_linear_scale_z (z1, Fznl.params);
-      gboolean found = FALSE; 
+		if (_nc_powspec_mnl_halofit_linear_scale_z (pshf->zmaxnl, Fznl.params) > R_min)
+			pshf->znl = pshf->zmaxnl;
+		else
+		{
+			const gdouble step = 1.0;
+			gdouble z0 = 0.0;
+			gdouble z1 = z0 + step;
+			gdouble R0 = _nc_powspec_mnl_halofit_linear_scale_z (z0, Fznl.params);
+			gdouble R1 = _nc_powspec_mnl_halofit_linear_scale_z (z1, Fznl.params);
+			gboolean found = FALSE;
 
-      if (R0 < R_min)
-        g_error ("_nc_powspec_mnl_halofit_prepare_nl: linear universe or too large R_min, in the latter case increase k_max (R0 == % 21.15g, R_min == % 21.15g).", R0, R_min);
-      
-      while (R1 > R_min)
-      {
-        z0  = z1;
-        z1 += step;
-        R1 = _nc_powspec_mnl_halofit_linear_scale_z (z1, Fznl.params);
-        if (z1 > pshf->zmaxnl)
-        {
-          pshf->znl = pshf->zmaxnl;
-          found = TRUE;
-          break;
-        }
-      }
+			if (R0 < R_min)
+				g_error ("_nc_powspec_mnl_halofit_prepare_nl: linear universe or too large R_min, in the latter case increase k_max (R0 == % 21.15g, R_min == % 21.15g).", R0, R_min);
 
-      if (!found)
-      {
-        gint status;
-        gint iter = 0, max_iter = 20000;
+			while (R1 > R_min)
+			{
+				z0 = z1;
+				z1 += step;
+				R1 = _nc_powspec_mnl_halofit_linear_scale_z (z1, Fznl.params);
+				if (z1 > pshf->zmaxnl)
+				{
+					pshf->znl = pshf->zmaxnl;
+					found = TRUE;
+					break;
+				}
+			}
 
-        gsl_root_fsolver_set (pshf->priv->znl_solver, &Fznl, z0, z1);
-        do
-        {
-          iter++;
-          status = gsl_root_fsolver_iterate (pshf->priv->znl_solver);
+			if (!found)
+			{
+				gint status;
+				gint iter = 0, max_iter = 20000;
 
-          pshf->znl = gsl_root_fsolver_root (pshf->priv->znl_solver);
-          z0        = gsl_root_fsolver_x_lower (pshf->priv->znl_solver);
-          z1        = gsl_root_fsolver_x_upper (pshf->priv->znl_solver);
-          status    = gsl_root_test_interval (z0, z1, 0.0, 1.0e-3);
-        } while (status == GSL_CONTINUE && iter < max_iter);
+				gsl_root_fsolver_set (pshf->priv->znl_solver, &Fznl, z0, z1);
+				do
+				{
+					iter++;
+					status = gsl_root_fsolver_iterate (pshf->priv->znl_solver);
 
-        if (iter >= max_iter)
-          g_warning ("_nc_powspec_mnl_halofit_prepare_nl: maximum number of iteration reached (%u), giving up.", max_iter);
-      }
-    }
+					pshf->znl = gsl_root_fsolver_root (pshf->priv->znl_solver);
+					z0 = gsl_root_fsolver_x_lower (pshf->priv->znl_solver);
+					z1 = gsl_root_fsolver_x_upper (pshf->priv->znl_solver);
+					status = gsl_root_test_interval (z0, z1, 0.0, 1.0e-3);
+				} while (status == GSL_CONTINUE && iter < max_iter);
 
-    {
-      gsl_function F;
-      F.function = &_nc_powspec_mnl_halofit_linear_scale_z;
-      F.params = &vps;
+				if (iter >= max_iter)
+					g_warning ("_nc_powspec_mnl_halofit_prepare_nl: maximum number of iteration reached (%u), giving up.", max_iter);
+			}
+		}
 
-      ncm_spline_set_func (pshf->Rsigma, NCM_SPLINE_FUNCTION_SPLINE, &F, 0.0, pshf->znl, 0, pshf->reltol);
-    }
+		{
+			gsl_function F;
+			F.function = &_nc_powspec_mnl_halofit_linear_scale_z;
+			F.params = &vps;
+
+			ncm_spline_set_func (pshf->Rsigma, NCM_SPLINE_FUNCTION_SPLINE, &F, 0.0, pshf->znl, 0, pshf->reltol);
+		}
 	}
 
 	{
-		const guint len  = ncm_vector_len (pshf->Rsigma->xv);
-		NcmVector *neffv = ncm_vector_new (len);
-		NcmVector *Curv  = ncm_vector_new (len);
+		const guint len = ncm_vector_len (pshf->Rsigma->xv);
+		NcmVector* neffv = ncm_vector_new (len);
+		NcmVector* Curv = ncm_vector_new (len);
 
 		for (i = 0; i < len; i++)
 		{
@@ -500,21 +570,22 @@ _nc_powspec_mnl_halofit_prepare_nl (NcPowspecMNLHaloFit* pshf, NcmModel *model)
 
 				ncm_vector_set (neffv, i, -3.0 - d1);
 				ncm_vector_set (Curv, i, -d2);
-
+/*
 				printf ("# z = % 20.15g, R = % 20.15g | % 20.15g % 20.15g % 20.15g | %e %e %e\n",
 				        z, R,
 				        sigma2_0, d1, d2,
 				        fabs ((ncm_powspec_filter_eval_var (pshf->psml_gauss, z, R) - sigma2_0) / sigma2_0),
 				        fabs ((ncm_powspec_filter_eval_dlnvar_dlnr (pshf->psml_gauss, z, log (R)) - d1) / d1),
 				        fabs ((ncm_powspec_filter_eval_dnlnvar_dlnrn (pshf->psml_gauss, z, log (R), 2) - d2) / d2));
-			}
+*/
+      }
 			else
 			{
-				const gdouble z   = ncm_vector_get (pshf->Rsigma->xv, i);
-				const gdouble R   = ncm_vector_get (pshf->Rsigma->yv, i);
+				const gdouble z = ncm_vector_get (pshf->Rsigma->xv, i);
+				const gdouble R = ncm_vector_get (pshf->Rsigma->yv, i);
 				const gdouble lnR = log (R);
-				const gdouble d1  = ncm_powspec_filter_eval_dlnvar_dlnr (pshf->psml_gauss, z, lnR);
-				const gdouble d2  = ncm_powspec_filter_eval_dnlnvar_dlnrn (pshf->psml_gauss, z, lnR, 2);
+				const gdouble d1 = ncm_powspec_filter_eval_dlnvar_dlnr (pshf->psml_gauss, z, lnR);
+				const gdouble d2 = ncm_powspec_filter_eval_dnlnvar_dlnrn (pshf->psml_gauss, z, lnR, 2);
 
 				ncm_vector_set (neffv, i, -3.0 - d1);
 				ncm_vector_set (Curv, i, -d2);
@@ -530,7 +601,7 @@ _nc_powspec_mnl_halofit_prepare_nl (NcPowspecMNLHaloFit* pshf, NcmModel *model)
 }
 
 static void
-_nc_powspec_mnl_halofit_prepare (NcmPowspec* powspec, NcmModel *model)
+_nc_powspec_mnl_halofit_prepare (NcmPowspec* powspec, NcmModel* model)
 {
 	NcPowspecMNLHaloFit* pshf = NC_POWSPEC_MNL_HALOFIT (powspec);
 
@@ -541,94 +612,201 @@ _nc_powspec_mnl_halofit_prepare (NcmPowspec* powspec, NcmModel *model)
 	_nc_powspec_mnl_halofit_prepare_nl (pshf, model);
 }
 
+static gdouble
+_nc_powspec_mnl_halofit_xi_cmp (gdouble w, gpointer params)
+{
+	var_params *vps = (var_params *)params;
+
+  ncm_model_orig_param_set (NCM_MODEL (vps->cosmo), NC_HICOSMO_DE_CPL_W0, w);
+  /*ncm_model_params_log_all (NCM_MODEL (vps->cosmo));*/
+
+  {
+    gdouble zdrag = nc_distance_drag_redshift (vps->pshf->priv->cpl_dist, vps->cosmo);
+    gdouble xi    = nc_distance_comoving (vps->pshf->priv->cpl_dist, vps->cosmo, zdrag) - nc_distance_comoving (vps->pshf->priv->cpl_dist, vps->cosmo, vps->z);
+
+    /*printf ("CMP % 22.15g % 22.15g | % 22.15g % 22.15g\n", vps->z, zdrag, xi, vps->R_min);*/
+
+    return xi / vps->R_min - 1.0;
+  }
+}
+
+
 static void
 _nc_powspec_mnl_halofit_preeval (NcPowspecMNLHaloFit* pshf, NcHICosmo* cosmo, const gdouble z)
 {
-	const gdouble E2     = nc_hicosmo_E2 (cosmo, z);
+	const gdouble E2 = nc_hicosmo_E2 (cosmo, z);
 	const gdouble Rsigma = ncm_spline_eval (pshf->Rsigma, z);
 
-	const gdouble neff   = ncm_spline_eval (pshf->neff, z);
-	const gdouble Cur    = ncm_spline_eval (pshf->Cur, z);
+	const gdouble neff = ncm_spline_eval (pshf->neff, z);
+	const gdouble Cur = ncm_spline_eval (pshf->Cur, z);
 
-	const gdouble neff2  = neff * neff;
-	const gdouble neff3  = neff2 * neff;
-	const gdouble neff4  = neff2 * neff2;
+	const gdouble neff2 = neff * neff;
+	const gdouble neff3 = neff2 * neff;
+	const gdouble neff4 = neff2 * neff2;
 
-	const gdouble Omega_de_onepp = NC_IS_HICOSMO_DE (cosmo) ? nc_hicosmo_de_E2Omega_de_onepw (NC_HICOSMO_DE (cosmo), z) / E2 : 0.0;
-	const gdouble Omega_m        = nc_hicosmo_Omega_m0 (cosmo) * gsl_pow_3 (1.0 + z) / E2;
+	const gdouble Omega_m = nc_hicosmo_E2Omega_m (cosmo, z) / E2;
+	const gdouble fnu = nc_hicosmo_E2Omega_mnu (cosmo, z) / nc_hicosmo_E2Omega_m (cosmo, z);
+	const gdouble frac = nc_hicosmo_de_E2Omega_de (NC_HICOSMO_DE (cosmo), z) / (E2 - nc_hicosmo_E2Omega_m (cosmo, z));
+	gdouble Omega_de_onepw = 0.0;
 
+  if (NC_IS_HICOSMO_DE (cosmo))
+  {
+    gdouble wa;
+    if (pshf->priv->pkequal && NC_IS_HICOSMO_DE_CPL (cosmo) && ((wa = ncm_model_orig_param_get (NCM_MODEL (cosmo), NC_HICOSMO_DE_CPL_W1)) != 0.0) )
+    {
+      /*printf ("Using pkequal!\n");*/
+      if (pshf->priv->cpl == NULL)
+      {
+        pshf->priv->cpl      = NC_HICOSMO (nc_hicosmo_de_cpl_new ());
+        pshf->priv->cpl_dist = nc_distance_new (1.0);
+      }
+
+      ncm_vector_memcpy (ncm_model_orig_params_peek_vector (NCM_MODEL (pshf->priv->cpl)), ncm_model_orig_params_peek_vector (NCM_MODEL (cosmo)));
+      ncm_model_orig_param_set (NCM_MODEL (pshf->priv->cpl), NC_HICOSMO_DE_CPL_W1, 0.0);
+      
+      {
+        gdouble zdrag_cosmo, xi_cosmo; 
+        nc_distance_prepare (pshf->priv->cpl_dist, cosmo);
+
+        zdrag_cosmo = nc_distance_drag_redshift (pshf->priv->cpl_dist, cosmo);
+        xi_cosmo    = nc_distance_comoving (pshf->priv->cpl_dist, cosmo, zdrag_cosmo) - nc_distance_comoving (pshf->priv->cpl_dist, cosmo, z);
+
+        {
+          var_params vps = { pshf, pshf->priv->cpl, z, xi_cosmo };
+
+          gsl_function Fxi;
+
+          gint status;
+          gint iter = 0, max_iter = 20000;
+          gdouble w0 = -2.0, w1 = 0.0, wfinal = 0.0;
+
+          Fxi.function = &_nc_powspec_mnl_halofit_xi_cmp;
+          Fxi.params = &vps;
+          
+          gsl_root_fsolver_set (pshf->priv->znl_solver, &Fxi, w0, w1);
+          do
+          {
+            iter++;
+            status = gsl_root_fsolver_iterate (pshf->priv->znl_solver);
+
+            wfinal = gsl_root_fsolver_root (pshf->priv->znl_solver);
+            w0     = gsl_root_fsolver_x_lower (pshf->priv->znl_solver);
+            w1     = gsl_root_fsolver_x_upper (pshf->priv->znl_solver);
+            status = gsl_root_test_interval (w0, w1, 0.0, 1.0e-6);
+          } while (status == GSL_CONTINUE && iter < max_iter);
+
+          ncm_model_orig_param_set (NCM_MODEL (pshf->priv->cpl), NC_HICOSMO_DE_CPL_W0, wfinal);
+          if (iter >= max_iter)
+            g_warning ("_nc_powspec_mnl_halofit_preeval: maximum number of iteration reached (%u), giving up.", max_iter);
+/*
+          printf ("# for z = % 22.15g, using wequiv = % 22.15g vs (w0 = % 22.15g, w1 = % 22.15g)\n", 
+                  z, wfinal, 
+                  ncm_model_orig_param_get (NCM_MODEL (cosmo), NC_HICOSMO_DE_CPL_W0),
+                  ncm_model_orig_param_get (NCM_MODEL (cosmo), NC_HICOSMO_DE_CPL_W1)
+                  );
+*/          
+          Omega_de_onepw = nc_hicosmo_de_E2Omega_de_onepw (NC_HICOSMO_DE (pshf->priv->cpl), z) / nc_hicosmo_E2 (pshf->priv->cpl, z);
+        }        
+        
+      }
+    }
+    else
+    {
+      /*printf ("Not using pkequal!\n");*/
+      Omega_de_onepw = nc_hicosmo_de_E2Omega_de_onepw (NC_HICOSMO_DE (cosmo), z) / E2;
+    }
+  }
+  
 	pshf->priv->z = z;
 
 	pshf->priv->ksigma = 1.0 / Rsigma;
 
-	pshf->priv->an     = ncm_util_exp10 (1.5222 + 2.8553 * neff + 2.3706 * neff2 + 0.9903 * neff3 + 0.2250 * neff4 - 0.6038 * Cur + 0.1749 * Omega_de_onepp);
-	pshf->priv->bn     = ncm_util_exp10 (-0.5642 + 0.5864 * neff + 0.5716 * neff2 - 1.5474 * Cur + 0.2279 * Omega_de_onepp);
-	pshf->priv->cn     = ncm_util_exp10 (0.3698 + 2.0404 * neff + 0.8161 * neff2 + 0.5869 * Cur);
+	pshf->priv->an = ncm_util_exp10 (1.5222 + 2.8553 * neff + 2.3706 * neff2 + 0.9903 * neff3 + 0.2250 * neff4 - 0.6038 * Cur + 0.1749 * Omega_de_onepw);
+	pshf->priv->bn = ncm_util_exp10 (-0.5642 + 0.5864 * neff + 0.5716 * neff2 - 1.5474 * Cur + 0.2279 * Omega_de_onepw);
+	pshf->priv->cn = ncm_util_exp10 (0.3698 + 2.0404 * neff + 0.8161 * neff2 + 0.5869 * Cur);
 	pshf->priv->gamman = 0.1971 - 0.0843 * neff + 0.8460 * Cur;
 	pshf->priv->alphan = fabs (6.0835 + 1.3373 * neff - 0.1959 * neff2 - 5.5274 * Cur);
-	pshf->priv->betan  = 2.0379 - 0.7354 * neff + 0.3157 * neff2 + 1.2490 * neff3 + 0.3980 * neff4 - 0.1682 * Cur; // + fnu*(1.081 + 0.395*pow(rneff,2)
-	pshf->priv->nun    = ncm_util_exp10 (5.2105 + 3.6902 * neff);
+	pshf->priv->betan = 2.0379 - 0.7354 * neff + 0.3157 * neff2 + 1.2490 * neff3 + 0.3980 * neff4 - 0.1682 * Cur + fnu * (1.081 + 0.395 * neff2);
+	pshf->priv->nun = ncm_util_exp10 (5.2105 + 3.6902 * neff);
 
-	pshf->priv->f1     = pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F1POW);
-	pshf->priv->f2     = pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F2POW);
-	pshf->priv->f3     = pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F3POW);
+	pshf->priv->f1 = frac * pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F1bPOW) + (1.0 - frac) * pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F1aPOW);
+	pshf->priv->f2 = frac * pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F2bPOW) + (1.0 - frac) * pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F2aPOW);
+	pshf->priv->f3 = frac * pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F3bPOW) + (1.0 - frac) * pow (Omega_m, NC_POWSPEC_MNL_HALOFIT_F3aPOW);
+
+	pshf->priv->mnu_corr_halo = 1.0 + fnu * (0.977 - 18.015 * (nc_hicosmo_Omega_m0 (cosmo) - 0.3));
+	pshf->priv->fnu = fnu;
+}
+
+static gdouble _nc_powspec_mnl_halofit_Pklin2Pknln (NcPowspecMNLHaloFit* pshf, NcHICosmo* cosmo, const gdouble k, const gdouble Pklin)
+{
+	const gdouble kh2 = gsl_pow_2 (k / nc_hicosmo_h (cosmo));
+	const gdouble k3 = gsl_pow_3 (k);
+	const gdouble k3o2pi2 = k3 / ncm_c_2_pi_2 ();
+	const gdouble Delta_lin = k3o2pi2 * Pklin;
+
+	const gdouble y = k / pshf->priv->ksigma;
+
+	const gdouble Delta_lin_nu = Delta_lin * (1.0 + pshf->priv->fnu * 47.48 * kh2 / (1.0 + 1.5 * kh2));
+	const gdouble P_Q = Pklin * (pow (1.0 + Delta_lin_nu, pshf->priv->betan) / (1.0 + pshf->priv->alphan * Delta_lin_nu)) * exp (-y / 4.0 - y * y / 8.0);
+
+	const gdouble Delta_Hprime = pshf->priv->an * pow (y, 3.0 * pshf->priv->f1) / (1.0 + pshf->priv->bn * pow (y, pshf->priv->f2) + pow (pshf->priv->cn * pshf->priv->f3 * y, 3.0 - pshf->priv->gamman));
+	const gdouble Delta_H = Delta_Hprime / (1.0 + pshf->priv->nun / (y * y)) * pshf->priv->mnu_corr_halo;
+
+	const gdouble P_H = Delta_H / k3o2pi2;
+
+	// if (gsl_isnan (P_Q + P_H))
+	// {
+	// 	printf("%g %g %g %g %g %g %g %g %g %g\n", kh2, k3, k3o2pi2, Delta_lin, y, Delta_lin_nu, P_Q, Delta_Hprime, Delta_H, P_H);
+	// 	g_error("_xcor_limber_gsl_auto_int");
+	// }
+
+
+	return P_Q + P_H;
 }
 
 static gdouble
-_nc_powspec_mnl_halofit_eval (NcmPowspec* powspec, NcmModel *model, const gdouble z, const gdouble k)
+_nc_powspec_mnl_halofit_eval (NcmPowspec* powspec, NcmModel* model, const gdouble z, const gdouble k)
 {
 	NcHICosmo* cosmo = NC_HICOSMO (model);
-	NcPowspecMNLHaloFit* pshf  = NC_POWSPEC_MNL_HALOFIT (powspec);
-  const gboolean linscale    = (z       > pshf->znl); 
-  const gboolean applysmooth = (z + 1.0 > pshf->znl);
-  const gdouble zhf          = linscale ? pshf->znl : z;
-	const gdouble Pklin        = ncm_powspec_eval (NCM_POWSPEC (pshf->psml), model, z, k);
-  gdouble Pknln;
+	NcPowspecMNLHaloFit* pshf = NC_POWSPEC_MNL_HALOFIT (powspec);
+	const gboolean linscale = (z > pshf->znl);
+	const gboolean applysmooth = (z + 1.0 > pshf->znl);
+	const gdouble zhf = linscale ? pshf->znl : z;
+	const gdouble Pklin = ncm_powspec_eval (NCM_POWSPEC (pshf->psml), model, z, k);
+	gdouble Pknln;
 
-  if (zhf != pshf->priv->z)
-  {
-    _nc_powspec_mnl_halofit_preeval (pshf, cosmo, zhf);
-  }
-  {
-    const gdouble k3           = gsl_pow_3 (k);
-    const gdouble k3o2pi2      = k3 / ncm_c_2_pi_2 ();
-    const gdouble Delta_lin    = k3o2pi2 * Pklin;
-    const gdouble y            = k / pshf->priv->ksigma;
-    const gdouble P_Q          = Pklin * (pow (1.0 + Delta_lin, pshf->priv->betan) / (1.0 + pshf->priv->alphan * Delta_lin)) * exp (-y / 4.0 - y * y / 8.0);
+	if (zhf != pshf->priv->z)
+	{
+		_nc_powspec_mnl_halofit_preeval (pshf, cosmo, zhf);
+	}
 
-    const gdouble Delta_Hprime = pshf->priv->an * pow (y, 3.0 * pshf->priv->f1) / (1.0 + pshf->priv->bn * pow (y, pshf->priv->f2) + pow (pshf->priv->cn * pshf->priv->f3 * y, 3.0 - pshf->priv->gamman));
-    const gdouble Delta_H      = Delta_Hprime / (1.0 + pshf->priv->nun / (y * y));
+	Pknln = _nc_powspec_mnl_halofit_Pklin2Pknln (pshf, cosmo, k, Pklin);
 
-    const gdouble P_H          = Delta_H / k3o2pi2;
+	if (applysmooth)
+		Pknln = ncm_util_smooth_trans (Pknln, Pklin, pshf->znl, 1.0, z);
 
-    Pknln = P_Q + P_H;
-  }
-
-  if (applysmooth)
-    Pknln = ncm_util_smooth_trans (Pknln, Pklin, pshf->znl, 1.0, z);
-
-  return Pknln;
+	return Pknln;
 }
 
 static void
-_nc_powspec_mnl_halofit_eval_vec (NcmPowspec* powspec, NcmModel *model, const gdouble z, NcmVector* k, NcmVector* Pk)
+_nc_powspec_mnl_halofit_eval_vec (NcmPowspec* powspec, NcmModel* model, const gdouble z, NcmVector* k, NcmVector* Pk)
 {
-	NcHICosmo *cosmo           = NC_HICOSMO (model);
-	NcPowspecMNLHaloFit *pshf  = NC_POWSPEC_MNL_HALOFIT (powspec);
-  const gboolean linscale    = (z       > pshf->znl); 
-  const gboolean applysmooth = (z + 1.0 > pshf->znl);
-  const gdouble zhf          = linscale ? pshf->znl : z;
-  gdouble theta0, theta1;
+	NcHICosmo* cosmo = NC_HICOSMO (model);
+	NcPowspecMNLHaloFit* pshf = NC_POWSPEC_MNL_HALOFIT (powspec);
+	const gboolean linscale = (z > pshf->znl);
+	const gboolean applysmooth = (z + 1.0 > pshf->znl);
+	const gdouble zhf = linscale ? pshf->znl : z;
+	gdouble theta0, theta1;
 
 	ncm_powspec_eval_vec (NCM_POWSPEC (pshf->psml), model, z, k, Pk);
 
-  if (applysmooth)
-    ncm_util_smooth_trans_get_theta (pshf->znl, 1.0, z, &theta0, &theta1);
+	if (applysmooth)
+		ncm_util_smooth_trans_get_theta (pshf->znl, 1.0, z, &theta0, &theta1);
 
-  if (zhf != pshf->priv->z)
-  {
-    _nc_powspec_mnl_halofit_preeval (pshf, cosmo, zhf);
-  }
+	if (zhf != pshf->priv->z)
+	{
+		_nc_powspec_mnl_halofit_preeval (pshf, cosmo, zhf);
+	}
 
 	{
 		const guint len = ncm_vector_len (k);
@@ -636,24 +814,15 @@ _nc_powspec_mnl_halofit_eval_vec (NcmPowspec* powspec, NcmModel *model, const gd
 
 		for (i = 0; i < len; i++)
 		{
-			const gdouble ki           = ncm_vector_get (k, i);
-			const gdouble ki3          = gsl_pow_3 (ki);
-			const gdouble ki3o2pi2     = ki3 / ncm_c_2_pi_2 ();
-			const gdouble Pklin        = ncm_vector_get (Pk, i);
-			const gdouble Delta_lin    = ki3o2pi2 * Pklin;
-			const gdouble y            = ki / pshf->priv->ksigma;
-      
-			const gdouble P_Q          = Pklin * (pow (1.0 + Delta_lin, pshf->priv->betan) / (1.0 + pshf->priv->alphan * Delta_lin)) * exp (-y / 4.0 - y * y / 8.0);
-			const gdouble Delta_Hprime = pshf->priv->an * pow (y, 3.0 * pshf->priv->f1) / (1.0 + pshf->priv->bn * pow (y, pshf->priv->f2) + pow (pshf->priv->cn * pshf->priv->f3 * y, 3.0 - pshf->priv->gamman));
-			const gdouble Delta_H      = Delta_Hprime / (1.0 + pshf->priv->nun / (y * y));
-			const gdouble P_H          = Delta_H / ki3o2pi2;
+			const gdouble ki = ncm_vector_get (k, i);
+			const gdouble Pklin = ncm_vector_get (Pk, i);
 
-      const gdouble Pknln        = P_Q + P_H;
+			const gdouble Pknln = _nc_powspec_mnl_halofit_Pklin2Pknln (pshf, cosmo, ki, Pklin);
 
-      if (applysmooth)
-        ncm_vector_set (Pk, i, theta0 * Pknln + theta1 * Pklin);
-      else
-        ncm_vector_set (Pk, i, Pknln);
+			if (applysmooth)
+				ncm_vector_set (Pk, i, theta0 * Pknln + theta1 * Pklin);
+			else
+				ncm_vector_set (Pk, i, Pknln);
 		}
 	}
 }
@@ -661,7 +830,7 @@ _nc_powspec_mnl_halofit_eval_vec (NcmPowspec* powspec, NcmModel *model, const gd
 static void
 _nc_powspec_mnl_halofit_get_nknots (NcmPowspec* powspec, guint* Nz, guint* Nk)
 {
-	NcPowspecMNLHaloFit *pshf = NC_POWSPEC_MNL_HALOFIT (powspec);
+	NcPowspecMNLHaloFit* pshf = NC_POWSPEC_MNL_HALOFIT (powspec);
 	ncm_powspec_get_nknots (NCM_POWSPEC (pshf->psml), Nz, Nk);
 	*Nz = ncm_vector_len (pshf->Rsigma->xv);
 }
@@ -677,14 +846,14 @@ _nc_powspec_mnl_halofit_get_nknots (NcmPowspec* powspec, guint* Nz, guint* Nk)
  *
  * Returns: (transfer full): the newly created #NcPowspecMNLHaloFit.
  */
-NcPowspecMNLHaloFit *
+NcPowspecMNLHaloFit*
 nc_powspec_mnl_halofit_new (NcPowspecML* psml, gdouble zmaxnl, gdouble reltol)
 {
-  NcPowspecMNLHaloFit *pshf = g_object_new (NC_TYPE_POWSPEC_MNL_HALOFIT,
-                                            "power-spec", psml,
-                                            "zmaxnl", zmaxnl,
-                                            "reltol", reltol,
-                                            NULL);
+	NcPowspecMNLHaloFit* pshf = g_object_new (NC_TYPE_POWSPEC_MNL_HALOFIT,
+	                                          "power-spec", psml,
+	                                          "zmaxnl", zmaxnl,
+	                                          "reltol", reltol,
+	                                          NULL);
 
 	return pshf;
 }
@@ -696,8 +865,38 @@ nc_powspec_mnl_halofit_new (NcPowspecML* psml, gdouble zmaxnl, gdouble reltol)
  * Sets mode $k$ boundaries from the linear matter power spectrum.
  *
  */
-void nc_powspec_mnl_halofit_set_kbounds_from_ml (NcPowspecMNLHaloFit *pshf)
+void nc_powspec_mnl_halofit_set_kbounds_from_ml (NcPowspecMNLHaloFit* pshf)
 {
 	ncm_powspec_set_kmin (NCM_POWSPEC (pshf), ncm_powspec_get_kmin (NCM_POWSPEC (pshf->psml)));
 	ncm_powspec_set_kmax (NCM_POWSPEC (pshf), ncm_powspec_get_kmax (NCM_POWSPEC (pshf->psml)));
 }
+
+/**
+ * nc_powspec_mnl_halofit_pkequal:
+ * @pshf: a #NcPowspecMNLHaloFit
+ * @on: a boolean
+ *
+ * Whether to use PKEqual to adjust the HaloFit formula when using a #NcHICosmoDECpl
+ * model, see [Casarini et al. (2009)][XCasarini2009] and [Casarini et al. (2016)][XCasarini2016].
+ *
+ */
+void 
+nc_powspec_mnl_halofit_pkequal (NcPowspecMNLHaloFit *pshf, gboolean on)
+{
+  if (on)
+  {
+    if (!pshf->priv->pkequal)
+    {
+      pshf->priv->pkequal = on;
+      ncm_model_ctrl_force_update (NCM_POWSPEC (pshf)->ctrl);
+    }
+  }
+  else
+  {
+    if (pshf->priv->pkequal)
+    {
+      pshf->priv->pkequal = on;
+      ncm_model_ctrl_force_update (NCM_POWSPEC (pshf)->ctrl);
+    }
+  }
+}
