@@ -29,8 +29,29 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
+
+#ifndef NUMCOSMO_GIR_SCAN
+
 #include <nvector/nvector_serial.h>
 
+#if HAVE_SUNDIALS_MAJOR == 2
+#include <cvodes/cvodes_dense.h>
+#define SUN_DENSE_ACCESS DENSE_ELEM
+#define SUN_BAND_ACCESS BAND_ELEM
+#elif HAVE_SUNDIALS_MAJOR == 3
+#include <cvodes/cvodes_direct.h> 
+#endif
+
+#if HAVE_SUNDIALS_MAJOR == 3
+#include <sundials/sundials_matrix.h>
+#include <sunmatrix/sunmatrix_dense.h>
+#include <sunlinsol/sunlinsol_dense.h>
+#define SUN_DENSE_ACCESS SM_ELEMENT_D
+#define SUN_BAND_ACCESS SM_ELEMENT_D
+#endif 
+
+#endif /* NUMCOSMO_GIR_SCAN */
+
 G_BEGIN_DECLS
 
 #define NC_TYPE_HIPERT             (nc_hipert_get_type ())
@@ -58,6 +79,10 @@ struct _NcHIPert
   GObject parent_instance;
   gdouble alpha0;
   N_Vector y;
+#if HAVE_SUNDIALS_MAJOR == 3
+  SUNMatrix A;
+  SUNLinearSolver LS;
+#endif
   guint sys_size;
   gpointer cvode;
   gboolean cvode_init;
