@@ -120,7 +120,8 @@ typedef enum _NcHIPertBoltzmannVars
   NC_HIPERT_BOLTZMANN_THETA2,
   NC_HIPERT_BOLTZMANN_THETA_P0,
   NC_HIPERT_BOLTZMANN_THETA_P1,
-  NC_HIPERT_BOLTZMANN_THETA_P2, /*< private >*/
+  NC_HIPERT_BOLTZMANN_THETA_P2, 
+  /* < private > */
   NC_HIPERT_BOLTZMANN_LEN,      /*< skip >*/
 } NcHIPertBoltzmannVars;
 
