@@ -37,17 +37,19 @@
  *
  */
 
-/*
- * It must be include before anything else, several symbols clash
- * with the default includes.
- */
-#include "class/include/class.h"
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 #include "build_cfg.h"
 
+/*
+ * It must be include before anything else, several symbols clash
+ * with the default includes.
+ */
+#ifndef NUMCOSMO_GIR_SCAN
+#include "class/include/class.h"
+#endif /* NUMCOSMO_GIR_SCAN */
+
 #include "nc_hiprim.h"
 #include "model/nc_hicosmo_de.h"
 #include "nc_hipert_boltzmann_cbe.h"
