@@ -165,15 +165,17 @@ test_ncm_model_child_child_new (TestNcmModel *test, gconstpointer pdata)
 NcmReparam *
 _test_ncm_model_create_reparam (TestNcmModel *test)
 {
-  guint size = ncm_model_len (NCM_MODEL (test->tm));
-  NcmMatrix *T = ncm_matrix_new (size, size);
-  NcmVector *v = ncm_vector_new (size);
-  NcmReparamLinear *relin;
+  const guint size     = ncm_model_len (NCM_MODEL (test->tm));
+  NcmMatrix *T         = ncm_matrix_new (size, size);
+  NcmVector *v         = ncm_vector_new (size);
   NcmBootstrap *bstrap = ncm_bootstrap_sized_new (size);
-  NcmRNG *rng = ncm_rng_new (NULL);
+  NcmRNG *rng          = ncm_rng_seeded_new (NULL, g_test_rand_int ());
+  guint cdesc_n        = size / 10;
+  NcmReparamLinear *relin;
   guint i;
-  guint cdesc_n = size / 10;
-  if (cdesc_n == 0) cdesc_n = 1;
+  
+  if (cdesc_n == 0) 
+    cdesc_n = 1;
 
   ncm_bootstrap_remix (bstrap, rng);
 
@@ -252,7 +254,7 @@ test_ncm_model_test_new (TestNcmModel *test, gconstpointer pdata)
 
   g_assert_cmpstr (ncm_model_name (model), ==, test->name);
   g_assert_cmpstr (ncm_model_nick (model), ==, test->nick);
-  g_assert (ncm_model_impl (model) == 0);
+
   g_assert (ncm_model_peek_reparam (model) == test->reparam);
 }
 
@@ -378,18 +380,19 @@ void
 test_ncm_model_test_setget (TestNcmModel *test, gconstpointer pdata)
 {
   NcmModelTest *tm = test->tm;
-  NcmModel *model = NCM_MODEL (tm);
-  guint model_len = ncm_model_len (model);
-  NcmVector *tmp = ncm_vector_new (model_len);
+  NcmModel *model  = NCM_MODEL (tm);
+  guint model_len  = ncm_model_len (model);
+  NcmVector *tmp   = ncm_vector_new (model_len);
   guint i;
 
   /* ncm_model_param_get / ncm_model_param_set */
 
   for (i = 0; i < model_len; i++)
   {
-    gdouble lb = ncm_model_param_get_lower_bound (model, i);
-    gdouble ub = ncm_model_param_get_upper_bound (model, i);
+    const gdouble lb = ncm_model_param_get_lower_bound (model, i);
+    const gdouble ub = ncm_model_param_get_upper_bound (model, i);
     gdouble val;
+    
     while ((val = g_test_rand_double_range (lb, ub)))
     {
       if (val != ncm_model_param_get (model, i))
