@@ -62,6 +62,8 @@ main (gint argc, gchar *argv[])
   gchar *full_cmd_line = NULL;
   gchar *runconf_cmd_line = NULL;
   gboolean is_de = FALSE;
+  gboolean is_gcg = FALSE;
+  gboolean is_idem2 = FALSE;  
   NcmRNG *rng = ncm_rng_pool_get ("darkenergy");
   NcmMSetCatalog *mcat = NULL;
   NcmSerialize *ser = ncm_serialize_global ();
@@ -274,6 +276,10 @@ main (gint argc, gchar *argv[])
 
   if (g_type_is_a (G_OBJECT_TYPE (cosmo), NC_TYPE_HICOSMO_DE))
     is_de = TRUE;
+  else if (g_type_is_a (G_OBJECT_TYPE (cosmo), NC_TYPE_HICOSMO_GCG))
+    is_gcg = TRUE;
+  else if (g_type_is_a (G_OBJECT_TYPE (cosmo), NC_TYPE_HICOSMO_IDEM2))
+    is_idem2 = TRUE;
 
   if (de_model.help_names)
   {
@@ -288,25 +294,64 @@ main (gint argc, gchar *argv[])
 
   if (de_model.flat)
   {
-    if (!is_de)
+    if (is_de)
+    {
+      nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (cosmo));
+      ncm_model_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_OMEGA_X, 0.0);
+      ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FIXED);
+    }
+    else if (is_gcg)
+    {
+      nc_hicosmo_gcg_omega_x2omega_k (NC_HICOSMO_GCG (cosmo));
+      ncm_model_param_set (NCM_MODEL (cosmo), NC_HICOSMO_GCG_OMEGA_X, 0.0);
+      ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_GCG_OMEGA_X, NCM_PARAM_TYPE_FIXED);
+    }
+    else if (is_idem2)
+    {
+      nc_hicosmo_idem2_omega_x2omega_k (NC_HICOSMO_IDEM2 (cosmo));
+      ncm_model_param_set (NCM_MODEL (cosmo), NC_HICOSMO_IDEM2_OMEGA_X, 0.0);
+      ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_IDEM2_OMEGA_X, NCM_PARAM_TYPE_FIXED);
+    }
+    else
       g_error ("flat option is valid only for darkenergy models");
-    nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (cosmo));
-    ncm_model_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_OMEGA_X, 0.0);
-    ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FIXED);
   }
   else if (de_model.Omega_k)
   {
-    if (!is_de)
+    if (is_de)
+    {
+      nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (cosmo));
+      ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FREE);
+    }
+    else if (is_gcg)
+    {
+      nc_hicosmo_gcg_omega_x2omega_k (NC_HICOSMO_GCG (cosmo));
+      ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_GCG_OMEGA_X, NCM_PARAM_TYPE_FREE);
+    }
+    else if (is_idem2)
+    {
+      nc_hicosmo_idem2_omega_x2omega_k (NC_HICOSMO_IDEM2 (cosmo));
+      ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_IDEM2_OMEGA_X, NCM_PARAM_TYPE_FREE);
+    }    
+    else
       g_error ("omegak option is valid only for darkenergy models");
-    nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (cosmo));
-    ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FREE);
   }
 
   if (de_model.pos_Omega_x)
   {
-    if (!is_de)
+    if (is_de)
+    {
+      ncm_likelihood_priors_add_flat_param (lh, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, 0.0, HUGE_VAL, 1.0);
+    }
+    else if (is_gcg)
+    {
+      ncm_likelihood_priors_add_flat_param (lh, nc_hicosmo_id (), NC_HICOSMO_GCG_OMEGA_X, 0.0, HUGE_VAL, 1.0);
+    }
+    else if (is_idem2)
+    {
+      ncm_likelihood_priors_add_flat_param (lh, nc_hicosmo_id (), NC_HICOSMO_IDEM2_OMEGA_X, 0.0, HUGE_VAL, 1.0);
+    }
+    else
       g_error ("omegak > 0 option is valid only for darkenergy models");
-    ncm_likelihood_priors_add_flat_param (lh, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, 0.0, HUGE_VAL, 1.0);
   }
   
   if (de_data_simple.snia_id != NULL)
@@ -613,14 +658,17 @@ main (gint argc, gchar *argv[])
     ncm_dataset_resample (dset, fiduc, rng);
   }
 
-  de_fit.fisher        = (de_fit.fisher || (de_fit.nsigma_fisher != -1) || (de_fit.nsigma != -1) || (de_fit.onedim_cr != NULL));
+  de_fit.fisher        = !de_fit.fisher && ((de_fit.nsigma_fisher != -1) || (de_fit.nsigma != -1) || (de_fit.onedim_cr != NULL)) ? 1 : de_fit.fisher;
   de_fit.fit           = (de_fit.fit || de_fit.fisher);
   de_fit.save_best_fit = (de_fit.save_best_fit || de_fit.save_fisher);
 
   if (de_fit.fit)
   {
     ncm_fit_set_maxiter (fit, de_fit.max_iter);
-    ncm_fit_run (fit, de_fit.msg_level);
+    if (de_fit.restart)
+      ncm_fit_run_restart (fit, de_fit.msg_level, de_fit.restart_abstol, de_fit.restart_reltol, NULL, de_fit.save_mset);
+    else
+      ncm_fit_run (fit, de_fit.msg_level);
     
     ncm_fit_log_info (fit);
   }
@@ -642,8 +690,21 @@ main (gint argc, gchar *argv[])
 
   if (de_fit.fisher)
   {
-    ncm_fit_numdiff_m2lnL_covar (fit);
+    switch (de_fit.fisher)
+    {
+      case 1:
+        ncm_fit_obs_fisher (fit);
+        break;
+      case 2:
+        ncm_fit_fisher (fit);
+        break;
+      default:
+        g_assert_not_reached ();
+        break;
+    }
+
     ncm_fit_log_covar (fit);
+    
     if (de_fit.save_fisher)
     {
       FILE *f_MF;
@@ -653,8 +714,8 @@ main (gint argc, gchar *argv[])
       ncm_fit_fishermatrix_print (fit, f_MF, full_cmd_line);
       fclose (f_MF);
 
-      ncm_message ("#---------------------------------------------------------------------------------- \n", mfile);
-      ncm_message ("# FM file: %s \n", mfile);
+      ncm_message ("#---------------------------------------------------------------------------------- \n");
+      ncm_message ("# FM file: %s\n", mfile);
 
       g_free (mfile);
 
@@ -910,11 +971,12 @@ main (gint argc, gchar *argv[])
 
   if (de_fit.onedim_cr != NULL)
   {
-    while (de_fit.onedim_cr[0] != NULL)
+    gchar **onedim_cr = de_fit.onedim_cr;
+    while (onedim_cr[0] != NULL)
     {
-      const gchar *pname = de_fit.onedim_cr[0];
-      const NcmMSetPIndex *pi = ncm_mset_fparam_get_pi_by_name (mset, de_fit.onedim_cr[0]);
-      de_fit.onedim_cr = &de_fit.onedim_cr[1];
+      const gchar *pname = onedim_cr[0];
+      const NcmMSetPIndex *pi = ncm_mset_fparam_get_pi_by_name (mset, onedim_cr[0]);
+      onedim_cr = &onedim_cr[1];
 
       if (pi == NULL)
       {
@@ -1132,7 +1194,7 @@ main (gint argc, gchar *argv[])
 
   ncm_serialize_clear (&ser);
   ncm_mset_catalog_clear (&mcat);
-  ncm_serialize_global_reset ();
+  ncm_serialize_global_reset (FALSE);
   ncm_model_free (NCM_MODEL (cosmo));
   ncm_mset_free (mset);
   ncm_fit_free (fit);
